@@ -1,5 +1,5 @@
 /* OGMDvd - A wrapper library around libdvdread
- * Copyright (C) 2004-2008 Olivier Rolland <billl@users.sf.net>
+ * Copyright (C) 2004-2009 Olivier Rolland <billl@users.sourceforge.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -20,7 +20,6 @@
 #define __OGMDVD_DRIVE_CHOOSER_WIDGET_H__
 
 #include <ogmdvd-drive-chooser.h>
-#include <gtk/gtkcombobox.h>
 
 G_BEGIN_DECLS
 
@@ -45,7 +44,7 @@ struct _OGMDvdDriveChooserWidgetClass
   GtkComboBoxClass parent_class;
 };
 
-GtkType             ogmdvd_drive_chooser_widget_get_type  (void);
+GType               ogmdvd_drive_chooser_widget_get_type  (void);
 GtkWidget *         ogmdvd_drive_chooser_widget_new       (void);
 
 G_END_DECLS
