@@ -36,33 +36,7 @@
 #include "ogmdvd-reader.h"
 #include "ogmdvd-parser.h"
 
-static glong
-ogmdvd_time_to_msec (dvd_time_t *dtime)
-{
-  guint hour, min, sec, frames;
-  gfloat fps;
-
-  hour   = ((dtime->hour    & 0xf0) >> 4) * 10 + (dtime->hour    & 0x0f);
-  min    = ((dtime->minute  & 0xf0) >> 4) * 10 + (dtime->minute  & 0x0f);
-  sec    = ((dtime->second  & 0xf0) >> 4) * 10 + (dtime->second  & 0x0f);
-  frames = ((dtime->frame_u & 0x30) >> 4) * 10 + (dtime->frame_u & 0x0f);
-
-  if (((dtime->frame_u & 0xc0) >> 6) == 1)
-    fps = 25.0;
-  else
-    fps = 30000 / 1001.0;
-
-  return hour * 60 * 60 * 1000 + min * 60 * 1000 + sec * 1000 + (gfloat) frames * 1000.0 / fps;
-}
-
-static void
-ogmdvd_msec_to_time (glong msec, OGMDvdTime *dtime)
-{
-  dtime->hour = msec / (60 * 60 * 1000);
-  dtime->min = msec / (60 * 1000) % 60;
-  dtime->sec = msec / 1000 % 60;
-  dtime->frames = msec % 1000;
-}
+#include <glib/gi18n-lib.h>
 
 /**
  * ogmdvd_title_ref:
@@ -75,7 +49,7 @@ ogmdvd_title_ref (OGMDvdTitle *title)
 {
   g_return_if_fail (title != NULL);
 
-  title->ref ++;
+  ogmdvd_disc_ref (title->disc);
 }
 
 /**
@@ -89,36 +63,86 @@ ogmdvd_title_unref (OGMDvdTitle *title)
 {
   g_return_if_fail (title != NULL);
 
-  if (title->ref > 0)
+  ogmdvd_disc_unref (title->disc);
+}
+
+/**
+ * ogmdvd_title_open:
+ * @title: An #OGMDvdTitle
+ * @error: Location to store the error occuring, or NULL to ignore errors.
+ *
+ * Opens @title, opening the disc if needed.
+ *
+ * Returns: #TRUE on success
+ *
+ */
+gboolean
+ogmdvd_title_open (OGMDvdTitle *title, GError **error)
+{
+  g_return_val_if_fail (title != NULL, FALSE);
+  g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
+
+  title->close_disc = !ogmdvd_disc_is_open (title->disc);
+
+  if (!ogmdvd_disc_open (title->disc, error))
+    return FALSE;
+
+  title->vts_file = ifoOpen (title->disc->reader, title->title_set_nr);
+  if (!title->vts_file)
   {
-    title->ref --;
-
-    if (title->ref == 0)
-    {
-      title->disc->titles = g_slist_remove (title->disc->titles, title);
-
-      if (title->vts_file)
-        ifoClose (title->vts_file);
-      title->vts_file = NULL;
-
-      if (title->disc)
-        ogmdvd_disc_unref (title->disc);
-      title->disc = NULL;
-
-      if (title->bitrates)
-        g_free (title->bitrates);
-      title->bitrates = NULL;
-
-      g_free (title);
-    }
+    ogmdvd_disc_close (title->disc);
+    g_set_error (error, OGMDVD_DISC_ERROR, OGMDVD_DISC_ERROR_VTS, _("Cannot open video titleset"));
+    return FALSE;
   }
+
+  return TRUE;
+}
+
+/**
+ * ogmdvd_title_close:
+ * @title: A #OGMDvdTitle
+ *
+ * Closes @title.
+ */
+void
+ogmdvd_title_close (OGMDvdTitle *title)
+{
+  g_return_if_fail (title != NULL);
+
+  if (title->vts_file)
+  {
+    ifoClose (title->vts_file);
+    title->vts_file = NULL;
+  }
+
+  if (title->close_disc)
+  {
+    ogmdvd_disc_close (title->disc);
+    title->close_disc = FALSE;
+  }
+}
+
+/**
+ * ogmdvd_title_is_open:
+ * @title: An #OGMDvdTitle
+ *
+ * Returns whether the title is open or not.
+ *
+ * Returns: TRUE if the title is open
+ */
+gboolean
+ogmdvd_title_is_open (OGMDvdTitle *title)
+{
+  g_return_val_if_fail (title != NULL, FALSE);
+
+  return title->vts_file != NULL;
 }
 
 /**
  * ogmdvd_title_get_disc:
  * @title: An #OGMDvdTitle
  *
- * Returns the disc the #OGMDvdTitle was opened from.
+ * Returns the disc the #OGMDvdTitle was open from.
  *
  * Returns: The #OGMDvdDisc, or NULL
  */
@@ -146,6 +170,22 @@ ogmdvd_title_get_nr (OGMDvdTitle *title)
   return title->nr;
 }
 
+/**
+ * ogmdvd_title_get_ts_nr:
+ * @title: An #OGMDvdTitle
+ *
+ * Returns the titleset number.
+ *
+ * Returns: The titleset number, or -1
+ */
+gint
+ogmdvd_title_get_ts_nr (OGMDvdTitle *title)
+{
+  g_return_val_if_fail (title != NULL, -1);
+
+  return title->title_set_nr;
+}
+
 /**
  * ogmdvd_title_get_vts_size:
  * @title: An #OGMDvdTitle
@@ -157,28 +197,9 @@ ogmdvd_title_get_nr (OGMDvdTitle *title)
 gint64
 ogmdvd_title_get_vts_size (OGMDvdTitle *title)
 {
-  OGMDvdDisc *disc;
-  gint64 size, fullsize;
-  guint vts;
-
   g_return_val_if_fail (title != NULL, -1);
 
-  disc = ogmdvd_title_get_disc (title);
-
-  vts = disc->vmg_file ? disc->vmg_file->tt_srpt->title[title->nr].title_set_nr : 1;
-
-  fullsize  = _ogmdvd_get_ifo_size (disc, vts);
-  fullsize += _ogmdvd_get_bup_size (disc, vts);
-  fullsize += _ogmdvd_get_menu_size (disc, vts);
-
-  if (vts > 0)
-  {
-    if ((size = _ogmdvd_get_vob_size (disc, vts)) == 0)
-      return 0;
-    fullsize += size;
-  }
-
-  return fullsize;
+  return title->vts_size;
 }
 
 /**
@@ -195,23 +216,19 @@ ogmdvd_title_get_vts_size (OGMDvdTitle *title)
 gdouble
 ogmdvd_title_get_length (OGMDvdTitle *title, OGMDvdTime  *length)
 {
-  pgc_t *pgc;
+  dvd_time_t *dtime = &title->playback_time;
 
   g_return_val_if_fail (title != NULL, -1.0);
 
-  pgc = title->vts_file->vts_pgcit->pgci_srp[title->pgcn - 1].pgc;
-
   if (length)
   {
-    dvd_time_t *dtime = &pgc->playback_time;
-
     length->hour   = ((dtime->hour    & 0xf0) >> 4) * 10 + (dtime->hour    & 0x0f);
     length->min    = ((dtime->minute  & 0xf0) >> 4) * 10 + (dtime->minute  & 0x0f);
     length->sec    = ((dtime->second  & 0xf0) >> 4) * 10 + (dtime->second  & 0x0f);
     length->frames = ((dtime->frame_u & 0x30) >> 4) * 10 + (dtime->frame_u & 0x0f);
   }
 
-  return ogmdvd_time_to_msec (&pgc->playback_time) / 1000.0;
+  return ogmdvd_time_to_msec (dtime) / 1000.0;
 }
 
 /**
@@ -230,35 +247,20 @@ ogmdvd_title_get_length (OGMDvdTitle *title, OGMDvdTime  *length)
 gdouble
 ogmdvd_title_get_chapters_length (OGMDvdTitle *title, guint start, gint end, OGMDvdTime *length)
 {
-  pgc_t *pgc;
-  guint pgn, pgcn;
-  glong last, cell, total;
+  gulong total;
 
   g_return_val_if_fail (title != NULL, -1.0);
+  g_return_val_if_fail (start < title->nr_of_programs, -1.0);
   g_return_val_if_fail (end < 0 || start <= end, -1.0);
 
-  pgc = title->vts_file->vts_pgcit->pgci_srp[title->pgcn - 1].pgc;
+  if (end < 0)
+    end = title->nr_of_programs - 1;
 
-  g_return_val_if_fail (start < pgc->nr_of_programs, -1.0);
-
-  pgcn = title->vts_file->vts_ptt_srpt->title[title->ttn - 1].ptt[start].pgcn;
-  pgc = title->vts_file->vts_pgcit->pgci_srp[pgcn - 1].pgc;
-
-  pgn = title->vts_file->vts_ptt_srpt->title[title->ttn - 1].ptt[start].pgn;
-  cell = pgc->program_map[pgn - 1] - 1;
-
-  last = pgc->nr_of_cells;
-  if (end > -1 && end < pgc->nr_of_programs - 1)
-  {
-    pgn = title->vts_file->vts_ptt_srpt->title[title->ttn - 1].ptt[end + 1].pgn;
-    last = pgc->program_map[pgn - 1] - 1;
-  }
-
-  if (start == 0 && last == pgc->nr_of_cells)
+  if (start == 0 && end + 1 == title->nr_of_programs)
     return ogmdvd_title_get_length (title, length);
 
-  for (total = 0; cell < last; cell ++)
-    total += ogmdvd_time_to_msec (&pgc->cell_playback[cell].playback_time);
+  for (total = 0; start <= end; start ++)
+    total += title->length_of_programs[start];
 
   if (length)
     ogmdvd_msec_to_time (total, length);
@@ -277,15 +279,11 @@ ogmdvd_title_get_chapters_length (OGMDvdTitle *title, guint start, gint end, OGM
 void
 ogmdvd_title_get_framerate (OGMDvdTitle *title, guint *numerator, guint *denominator)
 {
-  pgc_t *pgc;
-
   g_return_if_fail (title != NULL);
   g_return_if_fail (numerator != NULL);
   g_return_if_fail (denominator != NULL);
 
-  pgc = title->vts_file->vts_pgcit->pgci_srp[title->pgcn - 1].pgc;
-
-  switch ((pgc->playback_time.frame_u & 0xc0) >> 6)
+  switch ((title->playback_time.frame_u & 0xc0) >> 6)
   {
     case 1:
       *numerator = 25;
@@ -318,10 +316,10 @@ ogmdvd_title_get_size (OGMDvdTitle *title, guint *width, guint *height)
 
   *width = 0;
   *height = 480;
-  if (title->vts_file->vtsi_mat->vts_video_attr.video_format != 0)
+  if (title->video_format != 0)
     *height = 576;
 
-  switch (title->vts_file->vtsi_mat->vts_video_attr.picture_size)
+  switch (title->picture_size)
   {
     case 0:
       *width = 720;
@@ -357,7 +355,7 @@ ogmdvd_title_get_aspect_ratio  (OGMDvdTitle *title, guint *numerator, guint *den
   g_return_if_fail (numerator != NULL);
   g_return_if_fail (denominator != NULL);
 
-  switch (title->vts_file->vtsi_mat->vts_video_attr.display_aspect_ratio)
+  switch (title->display_aspect_ratio)
   {
     case 0:
       *numerator = 4;
@@ -387,7 +385,7 @@ ogmdvd_title_get_video_format (OGMDvdTitle *title)
 {
   g_return_val_if_fail (title != NULL, -1);
 
-  return title->vts_file->vtsi_mat->vts_video_attr.video_format;
+  return title->video_format;
 }
 
 /**
@@ -403,7 +401,7 @@ ogmdvd_title_get_display_aspect (OGMDvdTitle *title)
 {
   g_return_val_if_fail (title != NULL, -1);
 
-  switch (title->vts_file->vtsi_mat->vts_video_attr.display_aspect_ratio)
+  switch (title->display_aspect_ratio)
   {
     case 0:
       return OGMDVD_DISPLAY_ASPECT_4_3;
@@ -428,7 +426,7 @@ ogmdvd_title_get_display_format (OGMDvdTitle *title)
 {
   g_return_val_if_fail (title != NULL, -1);
 
-  return title->vts_file->vtsi_mat->vts_video_attr.permitted_df;
+  return title->permitted_df;
 }
 
 /**
@@ -442,15 +440,9 @@ ogmdvd_title_get_display_format (OGMDvdTitle *title)
 G_CONST_RETURN guint *
 ogmdvd_title_get_palette (OGMDvdTitle *title)
 {
-  pgc_t *pgc;
-  guint pgn;
-
   g_return_val_if_fail (title != NULL, NULL);
 
-  pgn = title->vts_file->vts_ptt_srpt->title[title->ttn - 1].ptt[0].pgn;
-  pgc = title->vts_file->vts_pgcit->pgci_srp[title->pgcn - 1].pgc;
-
-  return pgc->palette;
+  return title->palette;
 }
 
 /**
@@ -466,7 +458,7 @@ ogmdvd_title_get_n_angles (OGMDvdTitle *title)
 {
   g_return_val_if_fail (title != NULL, -1);
 
-  return title->disc->vmg_file ? title->disc->vmg_file->tt_srpt->title[title->nr].nr_of_angles : 1;
+  return title->nr_of_angles;
 }
 
 /**
@@ -480,13 +472,9 @@ ogmdvd_title_get_n_angles (OGMDvdTitle *title)
 gint
 ogmdvd_title_get_n_chapters (OGMDvdTitle *title)
 {
-  pgc_t *pgc;
-
   g_return_val_if_fail (title != NULL, -1);
 
-  pgc = title->vts_file->vts_pgcit->pgci_srp[title->pgcn - 1].pgc;
-
-  return pgc->nr_of_programs;
+  return title->nr_of_programs;
 }
 
 /**
@@ -500,33 +488,9 @@ ogmdvd_title_get_n_chapters (OGMDvdTitle *title)
 gint
 ogmdvd_title_get_n_audio_streams (OGMDvdTitle *title)
 {
-  gint i, naudio;
-
   g_return_val_if_fail (title != NULL, -1);
 
-  for (naudio = i = 0; i < title->vts_file->vtsi_mat->nr_of_vts_audio_streams; i++)
-    if (title->vts_file->vts_pgcit->pgci_srp[title->ttn - 1].pgc->audio_control[i] & 0x8000)
-      naudio ++;
-
-  return naudio;
-}
-
-static gint
-ogmdvd_title_remap_audio_nr (OGMDvdTitle *title, guint nr)
-{
-  guint i = 0, j = 0;
-
-  for (i = j = 0; i < title->vts_file->vtsi_mat->nr_of_vts_audio_streams; i++)
-  {
-    if (title->vts_file->vts_pgcit->pgci_srp[title->ttn - 1].pgc->audio_control[i] & 0x8000)
-    {
-      if (j == nr)
-        return i;
-      j ++;
-    }
-  }
-
-  return -1;
+  return title->nr_of_audio_streams;
 }
 
 static gint
@@ -547,37 +511,17 @@ ogmdvd_stream_find_by_nr (OGMDvdStream *stream, guint nr)
 OGMDvdAudioStream *
 ogmdvd_title_get_nth_audio_stream (OGMDvdTitle *title, guint nr)
 {
-  OGMDvdAudioStream *audio;
+  OGMDvdAudioStream *audio = NULL;
   GSList *link;
 
   g_return_val_if_fail (title != NULL, NULL);
-  g_return_val_if_fail (nr < title->vts_file->vtsi_mat->nr_of_vts_audio_streams, NULL);
+  g_return_val_if_fail (nr < title->nr_of_audio_streams, NULL);
 
   link = g_slist_find_custom (title->audio_streams, GUINT_TO_POINTER (nr), (GCompareFunc) ogmdvd_stream_find_by_nr);
   if (link)
   {
     audio = link->data;
-    audio->stream.ref ++;
-  }
-  else
-  {
-    gint real_nr;
-
-    real_nr = ogmdvd_title_remap_audio_nr (title, nr);
-    if (real_nr < 0)
-      return NULL;
-
-    ogmdvd_title_ref (title);
-
-    audio = g_new0 (OGMDvdAudioStream, 1);
-    audio->attr = &title->vts_file->vtsi_mat->vts_audio_attr[real_nr];
-    audio->stream.title = title;
-    audio->stream.nr = nr;
-    audio->stream.ref = 1;
-
-    audio->stream.id = title->vts_file->vts_pgcit->pgci_srp[title->ttn - 1].pgc->audio_control[real_nr] >> 8 & 7;
-
-    title->audio_streams = g_slist_append (title->audio_streams, audio);
+    ogmdvd_stream_ref (&audio->stream);
   }
 
   return audio;
@@ -589,25 +533,45 @@ ogmdvd_title_get_nth_audio_stream (OGMDvdTitle *title, guint nr)
  *
  * Returns a list of audio stream.
  *
- * Returns: The #GList, or NULL
+ * Returns: The #GSList, or NULL
  */
-GList *
+GSList *
 ogmdvd_title_get_audio_streams (OGMDvdTitle *title)
 {
-  OGMDvdAudioStream *stream;
-  GList *list = NULL;
-  guint nr;
-
   g_return_val_if_fail (title != NULL, NULL);
 
-  for (nr = 0; nr < title->vts_file->vtsi_mat->nr_of_vts_audio_streams; nr ++)
-  {
-    stream = ogmdvd_title_get_nth_audio_stream (title, nr);
-    if (stream)
-      list = g_list_append (list, stream);
-  }
+  g_slist_foreach (title->audio_streams, (GFunc) ogmdvd_stream_ref, NULL);
 
-  return list;
+  return g_slist_copy (title->audio_streams);;
+}
+
+/**
+ * ogmdvd_title_find_audio_stream:
+ * @title: An #OGMDvdTitle
+ * @func: A #GCompareFunc
+ * @data: The data to pass to @func
+ *
+ * Searches for an audio stream with custom criteria.
+ *
+ * Returns: An #OGMDvdAudioStream, or NULL
+ */
+OGMDvdAudioStream *
+ogmdvd_title_find_audio_stream (OGMDvdTitle *title, GCompareFunc func, gpointer data)
+{
+  OGMDvdAudioStream *audio = NULL;
+  GSList *link;
+
+  g_return_val_if_fail (title != NULL, NULL);
+  g_return_val_if_fail (func != NULL, NULL);
+
+  link = g_slist_find_custom (title->audio_streams, data, func);
+  if (link)
+  {
+    audio = link->data;
+    ogmdvd_stream_ref (&audio->stream);
+  };
+
+  return audio;
 }
 
 /**
@@ -621,33 +585,9 @@ ogmdvd_title_get_audio_streams (OGMDvdTitle *title)
 gint
 ogmdvd_title_get_n_subp_streams (OGMDvdTitle *title)
 {
-  gint i, nsubp;
-
   g_return_val_if_fail (title != NULL, -1);
 
-  for (nsubp = i = 0; i < title->vts_file->vtsi_mat->nr_of_vts_subp_streams; i++)
-    if (title->vts_file->vts_pgcit->pgci_srp[title->ttn - 1].pgc->subp_control[i] & 0x80000000)
-      nsubp ++;
-
-  return nsubp;
-}
-
-static gint
-ogmdvd_title_remap_subp_nr (OGMDvdTitle *title, guint nr)
-{
-  guint i = 0, j = 0;
-
-  for (i = j = 0; i < title->vts_file->vtsi_mat->nr_of_vts_subp_streams; i++)
-  {
-    if (title->vts_file->vts_pgcit->pgci_srp[title->ttn - 1].pgc->subp_control[i] & 0x80000000)
-    {
-      if (j == nr)
-        return i;
-      j ++;
-    }
-  }
-
-  return -1;
+  return title->nr_of_subp_streams;
 }
 
 /**
@@ -662,41 +602,17 @@ ogmdvd_title_remap_subp_nr (OGMDvdTitle *title, guint nr)
 OGMDvdSubpStream *
 ogmdvd_title_get_nth_subp_stream (OGMDvdTitle *title, guint nr)
 {
-  OGMDvdSubpStream *subp;
+  OGMDvdSubpStream *subp = NULL;
   GSList *link;
 
   g_return_val_if_fail (title != NULL, NULL);
-  g_return_val_if_fail (nr < title->vts_file->vtsi_mat->nr_of_vts_subp_streams, NULL);
+  g_return_val_if_fail (nr < title->nr_of_subp_streams, NULL);
 
   link = g_slist_find_custom (title->subp_streams, GUINT_TO_POINTER (nr), (GCompareFunc) ogmdvd_stream_find_by_nr);
   if (link)
   {
     subp = link->data;
-    subp->stream.ref ++;
-  }
-  else
-  {
-    gint real_nr;
-
-    real_nr = ogmdvd_title_remap_subp_nr (title, nr);
-    if (real_nr < 0)
-      return NULL;
-
-    ogmdvd_title_ref (title);
-
-    subp = g_new0 (OGMDvdSubpStream, 1);
-    subp->attr = &title->vts_file->vtsi_mat->vts_subp_attr[real_nr];
-    subp->stream.title = title;
-    subp->stream.ref = 1;
-    subp->stream.nr = nr;
-
-    subp->stream.id = nr;
-    if (title->vts_file->vtsi_mat->vts_video_attr.display_aspect_ratio == 0) /* 4:3 */
-      subp->stream.id = title->vts_file->vts_pgcit->pgci_srp[title->ttn - 1].pgc->subp_control[real_nr] >> 24 & 31;
-    else if (title->vts_file->vtsi_mat->vts_video_attr.display_aspect_ratio == 3) /* 16:9 */
-      subp->stream.id = title->vts_file->vts_pgcit->pgci_srp[title->ttn - 1].pgc->subp_control[real_nr] >> 8 & 31;
-
-    title->subp_streams = g_slist_append (title->subp_streams, subp);
+    ogmdvd_stream_ref (&subp->stream);
   }
 
   return subp;
@@ -708,25 +624,45 @@ ogmdvd_title_get_nth_subp_stream (OGMDvdTitle *title, guint nr)
  *
  * Returns a list of subp stream.
  *
- * Returns: The #GList, or NULL
+ * Returns: The #GSList, or NULL
  */
-GList *
+GSList *
 ogmdvd_title_get_subp_streams (OGMDvdTitle *title)
 {
-  OGMDvdSubpStream *stream;
-  GList *list = NULL;
-  guint nr;
-
   g_return_val_if_fail (title != NULL, NULL);
 
-  for (nr = 0; nr < title->vts_file->vtsi_mat->nr_of_vts_subp_streams; nr ++)
-  {
-    stream = ogmdvd_title_get_nth_subp_stream (title, nr);
-    if (stream)
-      list = g_list_append (list, stream);
-  }
+  g_slist_foreach (title->subp_streams, (GFunc) ogmdvd_stream_ref, NULL);
 
-  return list;
+  return g_slist_copy (title->subp_streams);
+}
+
+/**
+ * ogmdvd_title_find_subp_stream:
+ * @title: An #OGMDvdTitle
+ * @func: A #GCompareFunc
+ * @data: The data to pass to @func
+ *
+ * Searches for a subp stream with custom criteria.
+ *
+ * Returns: An #OGMDvdSubpStream, or NULL
+ */
+OGMDvdSubpStream *
+ogmdvd_title_find_subp_stream (OGMDvdTitle *title, GCompareFunc func, gpointer data)
+{
+  OGMDvdSubpStream *subp = NULL;
+  GSList *link;
+
+  g_return_val_if_fail (title != NULL, NULL);
+  g_return_val_if_fail (func != NULL, NULL);
+
+  link = g_slist_find_custom (title->subp_streams, data, func);
+  if (link)
+  {
+    subp = link->data;
+    ogmdvd_stream_ref (&subp->stream);
+  };
+
+  return subp;
 }
 
 /**
@@ -734,10 +670,10 @@ ogmdvd_title_get_subp_streams (OGMDvdTitle *title)
  * @title: An #OGMDvdTitle
  *
  * Performs a depper analysis of the title to get more information about it and
- * its audio and subtitle streams. This function should be called multipled timeѕ
+ * its audio and subtitle streams. This function should be called multiple times
  * until the analysis is complete.
  *
- * Returns: FALSE if the analysis if complete, TRUE otherwise
+ * Returns: FALSE if the analysis is complete, TRUE otherwise
  */
 gboolean
 ogmdvd_title_analyze (OGMDvdTitle *title)
