@@ -23,10 +23,13 @@
  * @short_description: A spawn that runs a command line
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include "ogmjob-exec.h"
 #include "ogmjob-log.h"
 
-#include "config.h"
 #include <glib/gi18n-lib.h>
 
 #include <sys/types.h>
@@ -187,7 +190,7 @@ ogmjob_exec_stderr_notify (OGMJobExec *exec)
 }
 
 static gboolean
-ogmjob_exec_watch (GIOChannel *channel, GIOCondition condition, OGMJobExec *exec)
+ogmjob_exec_watch (GIOChannel *channel, GIOCondition condition, OGMJobExec *exec, gboolean do_log)
 {
   GIOStatus status;
   gchar **vline, *buffer;
@@ -209,29 +212,37 @@ ogmjob_exec_watch (GIOChannel *channel, GIOCondition condition, OGMJobExec *exec
     return status == G_IO_STATUS_AGAIN;
   }
 
-  vline = g_strsplit (buffer, "\r\n", -1);
+  if (do_log)
+    ogmjob_log_write (buffer);
+#ifdef G_ENABLE_DEBUG
+  else
+    g_print ("%s", buffer);
+#endif
+
+  vline = g_strsplit_set (buffer, "\r\n", -1);
   g_free (buffer);
 
   if (vline)
   {
     for (i = 0; vline[i]; i++)
     {
-      ogmjob_log_write (vline[i]);
-
-      if (exec->priv->watch_func)
+      if (strlen (vline[i]) > 0)
       {
-        gdouble fraction;
-
-        fraction = exec->priv->watch_func (exec, vline[i], exec->priv->watch_data);
-        fraction = MIN (fraction, 1.0);
-
-        if (fraction > 0.0 && fraction > exec->priv->fraction + 0.001)
+        if (exec->priv->watch_func)
         {
-          exec->priv->fraction = fraction;
-          if (exec->priv->swapped)
-            g_signal_emit_by_name (exec->priv->watch_data, "progress", fraction);
-          else
-            g_signal_emit_by_name (exec, "progress", fraction);
+          gdouble fraction;
+
+          fraction = exec->priv->watch_func (exec, vline[i], exec->priv->watch_data);
+          fraction = MIN (fraction, 1.0);
+
+          if (fraction > 0.0 && fraction > exec->priv->fraction + 0.001)
+          {
+            exec->priv->fraction = fraction;
+            if (exec->priv->swapped)
+              g_signal_emit_by_name (exec->priv->watch_data, "progress", fraction);
+            else
+              g_signal_emit_by_name (exec, "progress", fraction);
+          }
         }
       }
     }
@@ -241,6 +252,18 @@ ogmjob_exec_watch (GIOChannel *channel, GIOCondition condition, OGMJobExec *exec
   return TRUE;
 }
 
+static gboolean
+ogmjob_exec_watch_stdout (GIOChannel *channel, GIOCondition condition, OGMJobExec *exec)
+{
+  return ogmjob_exec_watch (channel, condition, exec, FALSE);
+}
+
+static gboolean
+ogmjob_exec_watch_stderr (GIOChannel *channel, GIOCondition condition, OGMJobExec *exec)
+{
+  return ogmjob_exec_watch (channel, condition, exec, TRUE);
+}
+
 static gint
 ogmjob_exec_run (OGMJobSpawn *spawn)
 {
@@ -258,7 +281,7 @@ ogmjob_exec_run (OGMJobSpawn *spawn)
   ogmjob_log_write ("\n");
 
   exec->priv->error = NULL;
-  exec->priv->result = OGMJOB_RESULT_COMPLETED;
+  exec->priv->result = OGMJOB_RESULT_SUCCESS;
   exec->priv->status = 0;
 
   flags = G_SPAWN_SEARCH_PATH | G_SPAWN_DO_NOT_REAP_CHILD;
@@ -288,8 +311,8 @@ ogmjob_exec_run (OGMJobSpawn *spawn)
     channel = g_io_channel_unix_new (fdout);
     g_io_channel_set_close_on_unref (channel, TRUE);
     g_io_channel_set_encoding (channel, NULL, NULL);
-    exec->priv->src_out = g_io_add_watch_full (channel, G_PRIORITY_HIGH_IDLE,
-        G_IO_IN | G_IO_HUP | G_IO_ERR, (GIOFunc) ogmjob_exec_watch, exec, 
+    exec->priv->src_out = g_io_add_watch_full (channel, G_PRIORITY_DEFAULT_IDLE,
+        G_IO_IN | G_IO_HUP | G_IO_ERR, (GIOFunc) ogmjob_exec_watch_stdout, exec, 
         (GDestroyNotify) ogmjob_exec_stdout_notify);
     g_io_channel_unref (channel);
   }
@@ -300,8 +323,8 @@ ogmjob_exec_run (OGMJobSpawn *spawn)
     channel = g_io_channel_unix_new (fderr);
     g_io_channel_set_close_on_unref (channel, TRUE);
     g_io_channel_set_encoding (channel, NULL, NULL);
-    exec->priv->src_err = g_io_add_watch_full (channel, G_PRIORITY_HIGH_IDLE,
-        G_IO_IN | G_IO_HUP | G_IO_ERR, (GIOFunc) ogmjob_exec_watch, exec, 
+    exec->priv->src_err = g_io_add_watch_full (channel, G_PRIORITY_DEFAULT_IDLE,
+        G_IO_IN | G_IO_HUP | G_IO_ERR, (GIOFunc) ogmjob_exec_watch_stderr, exec, 
         (GDestroyNotify) ogmjob_exec_stderr_notify);
     g_io_channel_unref (channel);
   }
