@@ -29,6 +29,14 @@
 
 #include <string.h>
 
+#define OGMRIP_PLUGIN_DIALOG_GET_PRIVATE(o) \
+  (G_TYPE_INSTANCE_GET_PRIVATE ((o), OGMRIP_TYPE_PLUGIN_DIALOG, OGMRipPluginDialogPriv))
+
+struct _OGMRipPluginDialogPriv
+{
+  gchar *section;
+};
+
 typedef struct
 {
   GModule *module;
@@ -38,13 +46,155 @@ typedef struct
 
 typedef OGMRipOptionsPlugin * (* OGMRipOptionsPluginInit) (void);
 
+static void ogmrip_plugin_dialog_finalize     (GObject          *gobject);
+static void ogmrip_plugin_dialog_set_property (GObject          *gobject,
+                                               guint            property_id,
+                                               const GValue     *value,
+                                               GParamSpec       *pspec);
+static void ogmrip_plugin_dialog_get_property (GObject          *gobject,
+                                               guint            property_id,
+                                               GValue           *value,
+                                               GParamSpec       *pspec);
+enum
+{
+  PROP_0,
+  PROP_SECTION
+};
+
+enum
+{
+  SET_SECTION,
+  LAST_SIGNAL
+};
+
+static int signals[LAST_SIGNAL] = { 0 };
+
 static GSList *plugins = NULL;
 
-static GSList *
-ogmrip_options_plugin_load (const gchar *dirname)
-{
-  GSList *slist = NULL;
+G_DEFINE_ABSTRACT_TYPE (OGMRipPluginDialog, ogmrip_plugin_dialog, GTK_TYPE_DIALOG)
 
+static void
+ogmrip_plugin_dialog_class_init (OGMRipPluginDialogClass *klass)
+{
+  GObjectClass *gobject_class;
+
+  gobject_class = (GObjectClass *) klass;
+  gobject_class->finalize = ogmrip_plugin_dialog_finalize;
+  gobject_class->set_property = ogmrip_plugin_dialog_set_property;
+  gobject_class->get_property = ogmrip_plugin_dialog_get_property;
+
+  g_object_class_install_property (gobject_class, PROP_SECTION, 
+        g_param_spec_string ("section", "Section property", "Set the section property", 
+           NULL, G_PARAM_READWRITE));
+
+  /**
+   * OGMRipPluginDialog::set-section
+   * @section: the new section
+   *
+   * Emitted each a new section is set.
+   */
+  signals[SET_SECTION] = g_signal_new ("set-section", G_TYPE_FROM_CLASS (klass), 
+      G_SIGNAL_RUN_LAST | G_SIGNAL_NO_RECURSE | G_SIGNAL_NO_HOOKS,
+      G_STRUCT_OFFSET (OGMRipPluginDialogClass, set_section), NULL, NULL,
+      g_cclosure_marshal_VOID__STRING,
+      G_TYPE_NONE, 1, G_TYPE_STRING);
+
+  g_type_class_add_private (klass, sizeof (OGMRipPluginDialogPriv));
+}
+
+static void
+ogmrip_plugin_dialog_init (OGMRipPluginDialog *dialog)
+{
+  dialog->priv = OGMRIP_PLUGIN_DIALOG_GET_PRIVATE (dialog);
+}
+
+static void
+ogmrip_plugin_dialog_finalize (GObject *gobject)
+{
+  OGMRipPluginDialog *dialog;
+
+  dialog = OGMRIP_PLUGIN_DIALOG (gobject);
+
+  if (dialog->priv->section)
+  {
+    g_free (dialog->priv->section);
+    dialog->priv->section = NULL;
+  }
+
+  (*G_OBJECT_CLASS (ogmrip_plugin_dialog_parent_class)->finalize) (gobject);
+}
+
+static void
+ogmrip_plugin_dialog_set_property (GObject *gobject, guint property_id, const GValue *value, GParamSpec *pspec)
+{
+  switch (property_id) 
+  {
+    case PROP_SECTION:
+      ogmrip_plugin_dialog_set_section (OGMRIP_PLUGIN_DIALOG (gobject), g_value_get_string (value));
+      break;
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, property_id, pspec);
+      break;
+  }
+}
+
+static void
+ogmrip_plugin_dialog_get_property (GObject *gobject, guint property_id, GValue *value, GParamSpec *pspec)
+{
+  switch (property_id) 
+  {
+    case PROP_SECTION:
+      g_value_set_string (value, OGMRIP_PLUGIN_DIALOG (gobject)->priv->section);
+      break;
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, property_id, pspec);
+      break;
+  }
+}
+
+/**
+ * ogmrip_plugin_dialog_set_section:
+ * @dialog: an #OGMRipPluginDialog
+ * @section: a profile section
+ *
+ * Sets the profile.
+ */
+void
+ogmrip_plugin_dialog_set_section (OGMRipPluginDialog *dialog, const gchar *section)
+{
+  g_return_if_fail (OGMRIP_IS_PLUGIN_DIALOG (dialog));
+
+  if (dialog->priv->section)
+  {
+    g_free (dialog->priv->section);
+    dialog->priv->section = NULL;
+  }
+
+  if (section)
+    dialog->priv->section = g_strdup (section);
+
+  g_signal_emit (dialog, signals[SET_SECTION], 0, dialog->priv->section);
+}
+
+/**
+ * ogmrip_plugin_dialog_get_section:
+ * @dialog: an #OGMRipPluginDialog
+ *
+ * Gets the profile.
+ *
+ * Returns: the profile, or NULL
+ */
+gchar *
+ogmrip_plugin_dialog_get_section (OGMRipPluginDialog *dialog)
+{
+  g_return_val_if_fail (OGMRIP_IS_PLUGIN_DIALOG (dialog), NULL);
+
+  return g_strdup (dialog->priv->section);
+}
+
+static GSList *
+ogmrip_options_plugin_load (GSList *slist, const gchar *dirname)
+{
   GModule *module;
   GPatternSpec *pspec;
   GDir *dir;
@@ -116,6 +266,9 @@ ogmrip_options_plugin_load (const gchar *dirname)
   return slist;
 }
 
+#define OGMRIP_OPTIONS_PLUGINS_DIR \
+  OGMRIP_LIB_DIR G_DIR_SEPARATOR_S "ogmrip" G_DIR_SEPARATOR_S "options-plugins"
+
 /**
  * ogmrip_options_plugin_init:
  *
@@ -125,7 +278,15 @@ void
 ogmrip_options_plugin_init (void)
 {
   if (!plugins)
-    plugins = ogmrip_options_plugin_load (OGMRIP_LIB_DIR "/ogmrip/options-plugins");
+  {
+    gchar *dir;
+
+    plugins = ogmrip_options_plugin_load (plugins, OGMRIP_OPTIONS_PLUGINS_DIR);
+
+    dir = g_build_filename (g_get_home_dir (), ".ogmrip", "options-plugins", NULL);
+    plugins = ogmrip_options_plugin_load (plugins, dir);
+    g_free (dir);
+  }
 }
 
 static void
@@ -176,7 +337,8 @@ ogmrip_options_plugin_find_by_type (GType type)
 gboolean
 ogmrip_options_plugin_exists (GType type)
 {
-  g_return_val_if_fail (g_type_is_a (type, OGMRIP_TYPE_CONTAINER) ||
+  g_return_val_if_fail (type == G_TYPE_NONE ||
+      g_type_is_a (type, OGMRIP_TYPE_CONTAINER) ||
       g_type_is_a (type, OGMRIP_TYPE_CODEC), FALSE);
 
   if (ogmrip_options_plugin_find_by_type (type))
@@ -186,7 +348,7 @@ ogmrip_options_plugin_exists (GType type)
 }
 
 static GtkWidget *
-ogmrip_options_plugin_dialog_new (GType type, const gchar *profile)
+ogmrip_options_plugin_dialog_new (GType type, const gchar *section)
 {
   OGMRipOptionsPlugin *plugin;
 
@@ -194,154 +356,74 @@ ogmrip_options_plugin_dialog_new (GType type, const gchar *profile)
   if (!plugin)
     return NULL; 
 
-  return g_object_new (plugin->dialog, "key", profile, NULL);
+  return g_object_new (plugin->dialog, "section", section, NULL);
 }
 
 /**
  * ogmrip_container_options_plugin_dialog_new:
  * @type: The type of a container
- * @profile: The current profile
+ * @section: The section of the current profile
  *
  * Creates a new #GtkDialog to configure the container.
  *
  * Returns: a new #GtkDialog
  */
 GtkWidget *
-ogmrip_container_options_plugin_dialog_new (GType type, const gchar *profile)
+ogmrip_container_options_plugin_dialog_new (GType type, const gchar *section)
 {
   g_return_val_if_fail (g_type_is_a (type, OGMRIP_TYPE_CONTAINER), NULL);
 
-  return ogmrip_options_plugin_dialog_new (type, profile);
-}
-
-/**
- * ogmrip_container_options_plugin_set_options:
- * @container: The #OGMRipContainer to set the options to
- *
- * Propagates the options to @container.
- */
-void
-ogmrip_container_options_plugin_set_options (OGMRipContainer *container)
-{
-  OGMRipContainerOptionsPlugin *plugin;
-  GType type;
-
-  g_return_if_fail (OGMRIP_IS_CONTAINER (container));
-
-  type = G_TYPE_FROM_INSTANCE (container);
-  plugin = (OGMRipContainerOptionsPlugin* ) ogmrip_options_plugin_find_by_type (type);
-  if (plugin && plugin->set_options)
-    (* plugin->set_options) (container);
+  return ogmrip_options_plugin_dialog_new (type, section);
 }
 
 /**
  * ogmrip_video_options_plugin_dialog_new:
  * @type: The type of a video codec
- * @profile: The current profile
+ * @section: The section of the current profile
  *
  * Creates a new #GtkDialog to configure the codec.
  *
  * Returns: a new #GtkDialog
  */
 GtkWidget *
-ogmrip_video_options_plugin_dialog_new (GType type, const gchar *profile)
+ogmrip_video_options_plugin_dialog_new (GType type, const gchar *section)
 {
   g_return_val_if_fail (g_type_is_a (type, OGMRIP_TYPE_VIDEO_CODEC), NULL);
 
-  return ogmrip_options_plugin_dialog_new (type, profile);
-}
-
-/**
- * ogmrip_video_options_plugin_set_options:
- * @codec: The #OGMRipVideoCodec to set the options to
- *
- * Propagates the options to @codec.
- */
-void
-ogmrip_video_options_plugin_set_options (OGMRipVideoCodec *codec)
-{
-  OGMRipVideoOptionsPlugin *plugin;
-  GType type;
-
-  g_return_if_fail (OGMRIP_IS_VIDEO_CODEC (codec));
-
-  type = G_TYPE_FROM_INSTANCE (codec);
-  plugin = (OGMRipVideoOptionsPlugin* ) ogmrip_options_plugin_find_by_type (type);
-  if (plugin && plugin->set_options)
-    (* plugin->set_options) (codec);
+  return ogmrip_options_plugin_dialog_new (type, section);
 }
 
 /**
  * ogmrip_audio_options_plugin_dialog_new:
  * @type: The type of a audio codec
- * @profile: The current profile
+ * @section: The section of the current profile
  *
  * Creates a new #GtkDialog to configure the codec.
  *
  * Returns: a new #GtkDialog
  */
 GtkWidget *
-ogmrip_audio_options_plugin_dialog_new (GType type, const gchar *profile)
+ogmrip_audio_options_plugin_dialog_new (GType type, const gchar *section)
 {
   g_return_val_if_fail (g_type_is_a (type, OGMRIP_TYPE_AUDIO_CODEC), NULL);
 
-  return ogmrip_options_plugin_dialog_new (type, profile);
-}
-
-/**
- * ogmrip_audio_options_plugin_set_options:
- * @codec: The #OGMRipAudioCodec to set the options to
- *
- * Propagates the options to @codec.
- */
-void
-ogmrip_audio_options_plugin_set_options (OGMRipAudioCodec *codec)
-{
-  OGMRipAudioOptionsPlugin *plugin;
-  GType type;
-
-  g_return_if_fail (OGMRIP_IS_AUDIO_CODEC (codec));
-
-  type = G_TYPE_FROM_INSTANCE (codec);
-  plugin = (OGMRipAudioOptionsPlugin* ) ogmrip_options_plugin_find_by_type (type);
-  if (plugin && plugin->set_options)
-    (* plugin->set_options) (codec);
+  return ogmrip_options_plugin_dialog_new (type, section);
 }
 
 /**
  * ogmrip_subp_options_plugin_dialog_new:
  * @type: The type of a subp codec
- * @profile: The current profile
+ * @section: The section of the current profile
  *
  * Creates a new #GtkDialog to configure the codec.
  *
  * Returns: a new #GtkDialog
  */
 GtkWidget *
-ogmrip_subp_options_plugin_dialog_new (GType type, const gchar *profile)
+ogmrip_subp_options_plugin_dialog_new (GType type, const gchar *section)
 {
   g_return_val_if_fail (g_type_is_a (type, OGMRIP_TYPE_SUBP_CODEC), NULL);
 
-  return ogmrip_options_plugin_dialog_new (type, profile);
-}
-
-/**
- * ogmrip_subp_options_plugin_set_options:
- * @codec: The #OGMRipSubpCodec to set the options to
- *
- * Propagates the options to @codec.
- */
-void
-ogmrip_subp_options_plugin_set_options (OGMRipSubpCodec *codec)
-{
-  OGMRipSubpOptionsPlugin *plugin;
-  GType type;
-
-  g_return_if_fail (OGMRIP_IS_SUBP_CODEC (codec));
-
-  type = G_TYPE_FROM_INSTANCE (codec);
-  plugin = (OGMRipSubpOptionsPlugin* ) ogmrip_options_plugin_find_by_type (type);
-  if (plugin && plugin->set_options)
-    (* plugin->set_options) (codec);
+  return ogmrip_options_plugin_dialog_new (type, section);
 }
 
