@@ -40,15 +40,6 @@
 #include <string.h>
 
 #include <glib/gi18n-lib.h>
-#include <gtk/gtkcellrenderertext.h>
-#include <gtk/gtkfilechooserdialog.h>
-#include <gtk/gtkcelllayout.h>
-#include <gtk/gtkalignment.h>
-#include <gtk/gtkwindow.h>
-#include <gtk/gtklabel.h>
-#include <gtk/gtkstock.h>
-#include <gtk/gtkhbox.h>
-#include <gtk/gtkmain.h>
 
 #define OGMRIP_AUDIO_CHOOSER_WIDGET_GET_PRIVATE(o) \
   (G_TYPE_INSTANCE_GET_PRIVATE ((o), OGMRIP_TYPE_AUDIO_CHOOSER_WIDGET, OGMRipSourceChooserWidgetPriv))
@@ -172,8 +163,8 @@ static void ogmrip_source_chooser_widget_dialog_response   (OGMRipSourceChooserW
                                                             gint                      response,
                                                             GtkWidget                 *dialog);
 
-extern gchar *ogmdvd_languages[][3];
-extern guint  ogmdvd_nlanguages;
+extern const gchar *ogmdvd_languages[][3];
+extern const guint  ogmdvd_nlanguages;
 
 static gpointer ogmrip_audio_chooser_widget_parent_class = NULL;
 static gpointer ogmrip_subtitle_chooser_widget_parent_class = NULL;
