@@ -20,7 +20,7 @@
 #include "config.h"
 #endif
 
-#include "ogmrip-acopy.h"
+#include "ogmrip-audio-codec.h"
 #include "ogmrip-mplayer.h"
 #include "ogmrip-plugin.h"
 #include "ogmrip-version.h"
@@ -32,6 +32,25 @@
 
 #define ACOPY_SPF 1536
 
+#define OGMRIP_TYPE_AUDIO_COPY          (ogmrip_audio_copy_get_type ())
+#define OGMRIP_AUDIO_COPY(obj)          (G_TYPE_CHECK_INSTANCE_CAST ((obj), OGMRIP_TYPE_AUDIO_COPY, OGMRipAudioCopy))
+#define OGMRIP_AUDIO_COPY_CLASS(klass)  (G_TYPE_CHECK_CLASS_CAST ((klass), OGMRIP_TYPE_AUDIO_COPY, OGMRipAudioCopyClass))
+#define OGMRIP_IS_AUDIO_COPY(obj)       (G_TYPE_CHECK_INSTANCE_TYPE ((obj), OGMRIP_TYPE_AUDIO_COPY))
+#define OGMRIP_IS_AUDIO_COPY_CLASS(obj) (G_TYPE_CHECK_CLASS_TYPE ((klass), OGMRIP_TYPE_AUDIO_COPY))
+
+typedef struct _OGMRipAudioCopy      OGMRipAudioCopy;
+typedef struct _OGMRipAudioCopyClass OGMRipAudioCopyClass;
+
+struct _OGMRipAudioCopy
+{
+  OGMRipAudioCodec parent_instance;
+};
+
+struct _OGMRipAudioCopyClass
+{
+  OGMRipAudioCodecClass parent_class;
+};
+
 static gint ogmrip_audio_copy_run (OGMJobSpawn *spawn);
 static gint ogmrip_audio_copy_get_samples_per_frame (OGMRipAudioCodec *audio);
 
@@ -136,21 +155,6 @@ ogmrip_audio_copy_get_samples_per_frame (OGMRipAudioCodec *audio)
   return ACOPY_SPF;
 }
 
-/**
- * ogmrip_audio_copy_new:
- * @audio: An #OGMDvdAudioStream
- * @output: The output file
- *
- * Creates a new #OGMRipAudioCopy.
- *
- * Returns: The new #OGMRipAudioCopy
- */
-OGMJobSpawn *
-ogmrip_audio_copy_new (OGMDvdAudioStream *audio, const gchar *output)
-{
-  return g_object_new (OGMRIP_TYPE_AUDIO_COPY, "stream", audio, "output", output, NULL);
-}
-
 static OGMRipAudioPlugin acopy_plugin =
 {
   NULL,
