@@ -82,10 +82,15 @@ struct _OGMRipContainerClass
 
   /* vtable */
   gint (* get_overhead) (OGMRipContainer *container);
+  void (* set_options)  (OGMRipContainer *container,
+                         const gchar     *section);
 };
 
 GType              ogmrip_container_get_type             (void);
 
+void               ogmrip_container_set_options          (OGMRipContainer          *container,
+                                                          const gchar              *section);
+
 G_CONST_RETURN
 gchar *            ogmrip_container_get_output           (OGMRipContainer          *container);
 void               ogmrip_container_set_output           (OGMRipContainer          *container,
@@ -114,6 +119,8 @@ void               ogmrip_container_add_audio            (OGMRipContainer
                                                           OGMRipAudioCodec         *audio,
                                                           OGMRipAudioDemuxer       demuxer,
                                                           gint                     language);
+void               ogmrip_container_remove_audio         (OGMRipContainer          *container,
+                                                          OGMRipAudioCodec         *audio);
 GSList *           ogmrip_container_get_audio            (OGMRipContainer          *container);
 OGMRipAudioCodec * ogmrip_container_get_nth_audio        (OGMRipContainer          *container,
                                                           gint                     n);
@@ -126,6 +133,8 @@ void               ogmrip_container_add_subp             (OGMRipContainer
                                                           OGMRipSubpCodec          *subp,
                                                           OGMRipSubpDemuxer        demuxer,
                                                           gint                     language);
+void               ogmrip_container_remove_subp          (OGMRipContainer          *container,
+                                                          OGMRipSubpCodec          *subp);
 GSList *           ogmrip_container_get_subp             (OGMRipContainer          *container);
 gint               ogmrip_container_get_n_subp           (OGMRipContainer          *container);
 OGMRipSubpCodec *  ogmrip_container_get_nth_subp         (OGMRipContainer          *container,
@@ -137,6 +146,8 @@ void               ogmrip_container_foreach_subp         (OGMRipContainer
 void               ogmrip_container_add_chapters         (OGMRipContainer          *container,
                                                           OGMRipChapters           *chapters,
                                                           gint                     language);
+void               ogmrip_container_remove_chapters      (OGMRipContainer          *container,
+                                                          OGMRipChapters           *chapters);
 GSList *           ogmrip_container_get_chapters         (OGMRipContainer          *container);
 OGMRipChapters *   ogmrip_container_get_nth_chapters     (OGMRipContainer          *container,
                                                           gint                     n);
@@ -147,6 +158,8 @@ void               ogmrip_container_foreach_chapters     (OGMRipContainer
 
 void               ogmrip_container_add_file             (OGMRipContainer          *container,
                                                           OGMRipFile               *file);
+void               ogmrip_container_remove_file          (OGMRipContainer          *container,
+                                                          OGMRipFile               *file);
 GSList *           ogmrip_container_get_files            (OGMRipContainer          *container);
 OGMRipFile *       ogmrip_container_get_nth_file         (OGMRipContainer          *container,
                                                           gint                     n);
