@@ -29,6 +29,7 @@
 
 #include "ogmrip-file.h"
 #include "ogmrip-enums.h"
+#include "ogmrip-version.h"
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -47,6 +48,7 @@
 struct _OGMRipFile
 {
   gchar *filename;
+  gboolean do_unlink;
   gint format;
   gint type;
   gint lang;
@@ -84,7 +86,7 @@ struct _OGMRipSubpFile
 };
 
 static gchar **
-ogmrip_backend_identify_command (const gchar *filename)
+ogmrip_backend_identify_command (const gchar *filename, gboolean lavf)
 {
   GPtrArray *argv;
 
@@ -92,12 +94,28 @@ ogmrip_backend_identify_command (const gchar *filename)
 
   argv = g_ptr_array_new ();
   g_ptr_array_add (argv, g_strdup ("mplayer"));
+  g_ptr_array_add (argv, g_strdup ("-nolirc"));
+  g_ptr_array_add (argv, g_strdup ("-nocache"));
+
+  if (MPLAYER_CHECK_VERSION (1,0,3,0))
+  {
+    g_ptr_array_add (argv, g_strdup ("-noconfig"));
+    g_ptr_array_add (argv, g_strdup ("all"));
+  }
+
   g_ptr_array_add (argv, g_strdup ("-vo"));
   g_ptr_array_add (argv, g_strdup ("null"));
   g_ptr_array_add (argv, g_strdup ("-ao"));
   g_ptr_array_add (argv, g_strdup ("null"));
   g_ptr_array_add (argv, g_strdup ("-frames"));
   g_ptr_array_add (argv, g_strdup ("0"));
+
+  if (lavf)
+  {
+    g_ptr_array_add (argv, g_strdup ("-demuxer"));
+    g_ptr_array_add (argv, g_strdup ("lavf"));
+  }
+
   g_ptr_array_add (argv, g_strdup ("-identify"));
   g_ptr_array_add (argv, g_strdup (filename));
   g_ptr_array_add (argv, NULL);
@@ -114,6 +132,16 @@ ogmrip_backend_identify_sub_command (const gchar *filename, gboolean vobsub)
 
   argv = g_ptr_array_new ();
   g_ptr_array_add (argv, g_strdup ("mplayer"));
+  g_ptr_array_add (argv, g_strdup ("-nolirc"));
+  g_ptr_array_add (argv, g_strdup ("-nocache"));
+
+  if (MPLAYER_CHECK_VERSION (1,0,3,0))
+  {
+    g_ptr_array_add (argv, g_strdup ("-noconfig"));
+    g_ptr_array_add (argv, g_strdup ("all"));
+  }
+
+  g_ptr_array_add (argv, g_strdup ("-v"));
   g_ptr_array_add (argv, g_strdup ("/dev/zero"));
   g_ptr_array_add (argv, g_strdup ("-rawvideo"));
   g_ptr_array_add (argv, g_strdup ("pal:fps=25"));
@@ -253,12 +281,49 @@ ogmrip_file_unref (OGMRipFile *file)
     if (file->ref == 0)
     {
       close (file->fd);
+
+      if (file->do_unlink)
+        g_unlink (file->filename);
+
       g_free (file->filename);
       g_free (file);
     }
   }
 }
 
+/**
+ * ogmrip_file_set_unlink_on_unref:
+ * @file: An #OGMRipFile
+ * @do_unlink: Whether the file will be closed on the final unref of @file
+ *
+ * Sets whether the file will be unlinked when @file receives its final unref
+ * and is destroyed.
+ */
+void
+ogmrip_file_set_unlink_on_unref (OGMRipFile *file, gboolean do_unlink)
+{
+  g_return_if_fail (file != NULL);
+
+  file->do_unlink = do_unlink;
+}
+
+/**
+ * ogmrip_file_get_unlink_on_unref:
+ * @file: An #OGMRipFile
+ *
+ * Gets whether the file will be unlinked when @file receives its final unref
+ * and is destroyed.
+ *
+ * Returns: Whether the channel will be closed on the final unref
+ */
+gboolean
+ogmrip_file_get_unlink_on_unref (OGMRipFile *file)
+{
+  g_return_val_if_fail (file != NULL, FALSE);
+
+  return file->do_unlink;
+}
+
 /**
  * ogmrip_file_get_type:
  * @file: An #OGMRipFile
@@ -412,11 +477,11 @@ OGMRipFile *
 ogmrip_audio_file_new (const gchar *filename, GError **error)
 {
   OGMRipAudioFile *audio;
-  gchar **argv, *output, **lines;
-  gboolean is_video = FALSE;
 
   GError *tmp_error = NULL;
+  gboolean result, is_video = FALSE;
   gint i, rate = -1, bitrate = -1, format = -1, channels = -1;
+  gchar **argv, *output, **lines;
   gdouble length = -1.0;
 
   g_return_val_if_fail (filename != NULL, NULL);
@@ -424,15 +489,17 @@ ogmrip_audio_file_new (const gchar *filename, GError **error)
 
   g_return_val_if_fail (g_file_test (filename, G_FILE_TEST_IS_REGULAR), NULL);
 
-  argv = ogmrip_backend_identify_command (filename);
+  argv = ogmrip_backend_identify_command (filename, TRUE);
   if (!argv)
     return NULL;
 
-  if (!g_spawn_sync (NULL, argv, NULL, G_SPAWN_SEARCH_PATH | G_SPAWN_STDERR_TO_DEV_NULL,
-        NULL, NULL, &output, NULL, NULL, &tmp_error))
+  result = g_spawn_sync (NULL, argv, NULL, G_SPAWN_SEARCH_PATH | G_SPAWN_STDERR_TO_DEV_NULL,
+      NULL, NULL, &output, NULL, NULL, &tmp_error);
+  g_strfreev (argv);
+
+  if (!result)
   {
     g_propagate_error (error, tmp_error);
-    g_strfreev (argv);
     return NULL;
   }
 
@@ -670,11 +737,11 @@ static OGMRipSubpFile *
 ogmrip_subp_file_new_sub (const gchar *filename, GError **error)
 {
   OGMRipSubpFile *subp;
-  gchar **argv, *output, **lines;
-  gint format = -1;
 
   GError *tmp_error = NULL;
-  gint i, j;
+  gchar **argv, *output, **lines;
+  gint i, j, format = -1;
+  gboolean result;
 
   gchar *sr[] = 
   {
@@ -687,11 +754,13 @@ ogmrip_subp_file_new_sub (const gchar *filename, GError **error)
   if (!argv)
     return NULL;
 
-  if (!g_spawn_sync (NULL, argv, NULL, G_SPAWN_SEARCH_PATH | G_SPAWN_STDERR_TO_DEV_NULL,
-        NULL, NULL, &output, NULL, NULL, &tmp_error))
+  result = g_spawn_sync (NULL, argv, NULL, G_SPAWN_SEARCH_PATH | G_SPAWN_STDERR_TO_DEV_NULL,
+      NULL, NULL, &output, NULL, NULL, &tmp_error);
+  g_strfreev (argv);
+
+  if (!result)
   {
     g_propagate_error (error, tmp_error);
-    g_strfreev (argv);
     return NULL;
   }
 
@@ -883,27 +952,29 @@ OGMRipFile *
 ogmrip_video_file_new (const gchar *filename, GError **error)
 {
   OGMRipVideoFile *video;
-  gchar **argv, *output, **lines;
 
   GError *tmp_error = NULL;
   gint i, bitrate = -1, format = -1, width = -1, height = -1;
-  gdouble fps = -1.0, aspect = -1.0;
-  gdouble length = -1.0;
+  gdouble fps = -1.0, aspect = -1.0, length = -1.0;
+  gchar **argv, *output, **lines;
+  gboolean result;
 
   g_return_val_if_fail (filename != NULL, NULL);
   g_return_val_if_fail (error == NULL || *error == NULL, NULL);
 
   g_return_val_if_fail (g_file_test (filename, G_FILE_TEST_IS_REGULAR), NULL);
 
-  argv = ogmrip_backend_identify_command (filename);
+  argv = ogmrip_backend_identify_command (filename, FALSE);
   if (!argv)
     return NULL;
 
-  if (!g_spawn_sync (NULL, argv, NULL, G_SPAWN_SEARCH_PATH | G_SPAWN_STDERR_TO_DEV_NULL,
-        NULL, NULL, &output, NULL, NULL, &tmp_error))
+  result = g_spawn_sync (NULL, argv, NULL, G_SPAWN_SEARCH_PATH | G_SPAWN_STDERR_TO_DEV_NULL,
+      NULL, NULL, &output, NULL, NULL, &tmp_error);
+  g_strfreev (argv);
+
+  if (!result)
   {
     g_propagate_error (error, tmp_error);
-    g_strfreev (argv);
     return NULL;
   }
 
