@@ -652,28 +652,47 @@ ogmrip_fs_set_extension (const gchar *filename, const gchar *extension)
 
   dot = strrchr (filename, '.');
   if (!dot)
-    return g_strconcat (filename, extension, NULL);
   {
-    if (dot[1] == '\0')
-      return g_strconcat (filename, ".", extension, NULL);
+    if (*extension == '.')
+      return g_strconcat (filename, extension, NULL);
+
+    return g_strconcat (filename, ".", extension, NULL);
+  }
+
+  if (dot[1] == '\0')
+  {
+    if (*extension == '.')
+      return g_strconcat (filename, extension + 1, NULL);
+
+    return g_strconcat (filename, ".", extension, NULL);
+  }
+
+  if (strcmp (dot + 1, extension) == 0)
+    return g_strdup (filename);
+  else
+  {
+    gchar *name;
+
+    name = g_new0 (gchar, dot - filename + 5);
+    strncpy (name, filename, dot - filename + 1);
+
+    if (*extension == '.')
+      strcat (name, extension + 1);
     else
-    {
-      if (strcmp (dot + 1, extension) == 0)
-        return g_strdup (filename);
-      else
-      {
-        gchar *name;
+      strcat (name, extension);
 
-        name = g_new0 (gchar, dot - filename + 5);
-        strncpy (name, filename, dot - filename + 1);
-        strcat (name, extension);
-
-        return name;
-      }
-    }
+    return name;
   }
 }
 
+/**
+ * ogmrip_fs_get_full_path:
+ * @filename: The path to an existing filename
+ *
+ * Return the full absolute path of @filename.
+ * 
+ * Returns: the full path, or NULL
+ */
 gchar *
 ogmrip_fs_get_full_path (const gchar *filename)
 {
