@@ -20,13 +20,15 @@
 #include "config.h"
 #endif
 
+#include "ogmrip-container.h"
 #include "ogmrip-fs.h"
-#include "ogmrip-mkv.h"
 #include "ogmrip-version.h"
 #include "ogmrip-plugin.h"
 #include "ogmjob-exec.h"
 
+#include <errno.h>
 #include <stdio.h>
+#include <stdlib.h>
 #include <string.h>
 #include <glib/gstdio.h>
 #include <glib/gi18n-lib.h>
@@ -34,10 +36,31 @@
 #define PROGRAM "mkvmerge"
 #define MKV_OVERHEAD 4
 
-static gint ogmrip_matroska_run          (OGMJobSpawn     *spawn);
+#define OGMRIP_TYPE_MATROSKA          (ogmrip_matroska_get_type ())
+#define OGMRIP_MATROSKA(obj)          (G_TYPE_CHECK_INSTANCE_CAST ((obj), OGMRIP_TYPE_MATROSKA, OGMRipMatroska))
+#define OGMRIP_MATROSKA_CLASS(klass)  (G_TYPE_CHECK_CLASS_CAST ((klass), OGMRIP_TYPE_MATROSKA, OGMRipMatroskaClass))
+#define OGMRIP_IS_MATROSKA(obj)       (G_TYPE_CHECK_INSTANCE_TYPE ((obj), OGMRIP_TYPE_MATROSKA))
+#define OGMRIP_IS_MATROSKA_CLASS(obj) (G_TYPE_CHECK_CLASS_TYPE ((klass), OGMRIP_TYPE_MATROSKA))
+
+typedef struct _OGMRipMatroska      OGMRipMatroska;
+typedef struct _OGMRipMatroskaClass OGMRipMatroskaClass;
+
+struct _OGMRipMatroska
+{
+  OGMRipContainer parent_instance;
+};
+
+struct _OGMRipMatroskaClass
+{
+  OGMRipContainerClass parent_class;
+};
+
+GType ogmrip_matroska_get_type (void);
+static gint ogmrip_matroska_run (OGMJobSpawn *spawn);
 static gint ogmrip_matroska_get_overhead (OGMRipContainer *container);
 
 static gint major_version = 0;
+static gint minor_version = 0;
 
 static gdouble
 ogmrip_matroska_watch (OGMJobExec *exec, const gchar *buffer, OGMRipContainer *matroska)
@@ -84,8 +107,8 @@ ogmrip_matroska_get_sync (OGMRipContainer *container)
 }
 
 static void
-ogmrip_matroska_append_audio_file (OGMRipContainer *matroska,
-    const gchar *filename, gint language, GPtrArray *argv)
+ogmrip_matroska_append_audio_file (OGMRipContainer *matroska, const gchar *filename,
+    const gchar *label, gint language, GPtrArray *argv)
 {
   struct stat buf;
 
@@ -102,14 +125,15 @@ ogmrip_matroska_append_audio_file (OGMRipContainer *matroska,
       {
         g_ptr_array_add (argv, g_strdup ("--language"));
         g_ptr_array_add (argv, g_strconcat ("0:", iso639_2, NULL));
-        if (MPLAYER_CHECK_VERSION (1,0,0,8))
-        {
-          g_ptr_array_add (argv, g_strdup ("--language"));
-          g_ptr_array_add (argv, g_strconcat ("1:", iso639_2, NULL));
-        }
       }
     }
 
+    if (label)
+    {
+      g_ptr_array_add (argv, g_strdup ("--track-name"));
+      g_ptr_array_add (argv, g_strconcat ("0:", label, NULL));
+    }
+
     sync = ogmrip_matroska_get_sync (matroska);
     if (sync)
     {
@@ -127,7 +151,7 @@ ogmrip_matroska_append_audio_file (OGMRipContainer *matroska,
 
 static void
 ogmrip_matroska_append_subp_file (OGMRipContainer *matroska, const gchar *filename,
-    gint demuxer, gint charset, gint language, GPtrArray *argv)
+    const gchar *label, gint demuxer, gint charset, gint language, GPtrArray *argv)
 {
   gchar *real_filename;
   gboolean do_merge;
@@ -182,6 +206,12 @@ ogmrip_matroska_append_subp_file (OGMRipContainer *matroska, const gchar *filena
       }
     }
 
+    if (label)
+    {
+      g_ptr_array_add (argv, g_strdup ("--track-name"));
+      g_ptr_array_add (argv, g_strconcat ("0:", label, NULL));
+    }
+
     switch (charset)
     {
       case OGMRIP_CHARSET_UTF8:
@@ -211,22 +241,26 @@ static void
 ogmrip_matroska_foreach_audio (OGMRipContainer *matroska, 
     OGMRipCodec *codec, guint demuxer, gint language, GPtrArray *argv)
 {
-  const gchar *input;
+  const gchar *input, *label;
 
   input = ogmrip_codec_get_output (codec);
-  ogmrip_matroska_append_audio_file (matroska, input, language, argv);
+  label = ogmrip_audio_codec_get_label (OGMRIP_AUDIO_CODEC (codec));
+
+  ogmrip_matroska_append_audio_file (matroska, input, label, language, argv);
 }
 
 static void
 ogmrip_matroska_foreach_subp (OGMRipContainer *matroska, 
     OGMRipCodec *codec, guint demuxer, gint language, GPtrArray *argv)
 {
-  const gchar *input;
+  const gchar *input, *label;
   gint charset;
 
   input = ogmrip_codec_get_output (codec);
+  label = ogmrip_subp_codec_get_label (OGMRIP_SUBP_CODEC (codec));
   charset = ogmrip_subp_codec_get_charset (OGMRIP_SUBP_CODEC (codec));
-  ogmrip_matroska_append_subp_file (matroska, input, demuxer, charset, language, argv);
+
+  ogmrip_matroska_append_subp_file (matroska, input, label, demuxer, charset, language, argv);
 }
 
 static void
@@ -274,11 +308,11 @@ ogmrip_matroska_foreach_file (OGMRipContainer *matroska, OGMRipFile *file, GPtrA
     switch (ogmrip_file_get_type (file))
     {
       case OGMRIP_FILE_TYPE_AUDIO:
-        ogmrip_matroska_append_audio_file (matroska, filename, lang, argv);
+        ogmrip_matroska_append_audio_file (matroska, filename, NULL, lang, argv);
         break;
       case OGMRIP_FILE_TYPE_SUBP:
         charset = ogmrip_subp_file_get_charset (OGMRIP_SUBP_FILE (file));
-        ogmrip_matroska_append_subp_file (matroska, filename, OGMRIP_SUBP_DEMUXER_AUTO, charset, lang, argv);
+        ogmrip_matroska_append_subp_file (matroska, filename, NULL, OGMRIP_SUBP_DEMUXER_AUTO, charset, lang, argv);
         break;
       default:
         g_assert_not_reached ();
@@ -312,30 +346,32 @@ ogmrip_matroska_command (OGMRipContainer *matroska)
     g_ptr_array_add (argv, g_strconcat ("0:", fourcc, NULL));
   }
 
-  video = ogmrip_container_get_video (matroska);
-  if (major_version == 1)
+  if ((video = ogmrip_container_get_video (matroska)))
   {
-    if (ogmrip_plugin_get_video_codec_format (G_TYPE_FROM_INSTANCE (video)) == OGMRIP_FORMAT_H264)
+    if (major_version == 1)
     {
-      /*
-       * Option to merge h264 streams
-       */
-      g_ptr_array_add (argv, g_strdup ("--engage"));
-      g_ptr_array_add (argv, g_strdup ("allow_avc_in_vfw_mode"));
+      if (ogmrip_plugin_get_video_codec_format (G_TYPE_FROM_INSTANCE (video)) == OGMRIP_FORMAT_H264)
+      {
+        /*
+         * Option to merge h264 streams
+         */
+        g_ptr_array_add (argv, g_strdup ("--engage"));
+        g_ptr_array_add (argv, g_strdup ("allow_avc_in_vfw_mode"));
+      }
     }
+
+    g_ptr_array_add (argv, g_strdup ("--command-line-charset"));
+    g_ptr_array_add (argv, g_strdup ("UTF-8"));
+
+    filename = ogmrip_codec_get_output (OGMRIP_CODEC (video));
+
+    g_ptr_array_add (argv, g_strdup ("-d"));
+    g_ptr_array_add (argv, g_strdup ("0"));
+    g_ptr_array_add (argv, g_strdup ("-A"));
+    g_ptr_array_add (argv, g_strdup ("-S"));
+    g_ptr_array_add (argv, g_strdup (filename));
   }
 
-  g_ptr_array_add (argv, g_strdup ("--command-line-charset"));
-  g_ptr_array_add (argv, g_strdup ("UTF-8"));
-
-  filename = ogmrip_codec_get_output (OGMRIP_CODEC (video));
-
-  g_ptr_array_add (argv, g_strdup ("-d"));
-  g_ptr_array_add (argv, g_strdup ("0"));
-  g_ptr_array_add (argv, g_strdup ("-A"));
-  g_ptr_array_add (argv, g_strdup ("-S"));
-  g_ptr_array_add (argv, g_strdup (filename));
-
   ogmrip_container_foreach_audio (matroska, 
       (OGMRipContainerCodecFunc) ogmrip_matroska_foreach_audio, argv);
   ogmrip_container_foreach_subp (matroska, 
@@ -406,7 +442,7 @@ ogmrip_matroska_run (OGMJobSpawn *spawn)
    * If mkvmerge resturns 1, it's only a warning
    */
   if (ogmjob_exec_get_status (OGMJOB_EXEC (child)) == 1)
-    result = OGMJOB_RESULT_COMPLETED;
+    result = OGMJOB_RESULT_SUCCESS;
 
   ogmjob_container_remove (OGMJOB_CONTAINER (spawn), child);
 
@@ -419,28 +455,13 @@ ogmrip_matroska_get_overhead (OGMRipContainer *container)
   return MKV_OVERHEAD;
 }
 
-/**
- * ogmrip_matroska_new:
- * @output: The output file
- *
- * Creates a new #OGMRipMatroska.
- *
- * Returns: The new #OGMRipMatroska
- */
-OGMJobSpawn *
-ogmrip_matroska_new (const gchar *output)
-{
-  g_return_val_if_fail (output && *output, NULL);
-
-  return g_object_new (OGMRIP_TYPE_MATROSKA, "output", output, NULL);
-}
-
 static OGMRipContainerPlugin mkv_plugin =
 {
   NULL,
   G_TYPE_NONE,
   "mkv",
   N_("Matroska Media (MKV)"),
+  FALSE,
   TRUE,
   G_MAXINT,
   G_MAXINT,
@@ -454,10 +475,10 @@ static gint formats[] =
   OGMRIP_FORMAT_MPEG4,
   OGMRIP_FORMAT_H264,
   OGMRIP_FORMAT_THEORA,
-  OGMRIP_FORMAT_AAC,
   OGMRIP_FORMAT_AC3,
   OGMRIP_FORMAT_DTS,
   OGMRIP_FORMAT_COPY,
+  OGMRIP_FORMAT_AAC,
   OGMRIP_FORMAT_MP3,
   OGMRIP_FORMAT_VORBIS,
   OGMRIP_FORMAT_PCM,
@@ -468,15 +489,27 @@ static gint formats[] =
 };
 
 OGMRipContainerPlugin *
-ogmrip_init_plugin (void)
+ogmrip_init_plugin (GError **error)
 {
   gchar *output;
 
-  if (!g_spawn_command_line_sync ("mkvmerge --version", &output, NULL, NULL, NULL))
-    return NULL;
+  g_return_val_if_fail (error == NULL || *error == NULL, NULL);
 
-  if (strncmp (output, "mkvmerge v1", 11) == 0)
-    major_version = 1;
+  if (!g_spawn_command_line_sync ("mkvmerge --version", &output, NULL, NULL, NULL))
+  {
+    g_set_error (error, OGMRIP_PLUGIN_ERROR, OGMRIP_PLUGIN_ERROR_REQ, _("mkvmerge is missing"));
+    return NULL;
+  }
+
+  if (strncmp (output, "mkvmerge v", 10) == 0)
+  {
+    gchar *end;
+
+    errno = 0;
+    major_version = strtoul (output + 10, &end, 10);
+    if (!errno && *end == '.')
+      minor_version = strtoul (end + 1, NULL, 10);
+  }
 
   mkv_plugin.type = OGMRIP_TYPE_MATROSKA;
   mkv_plugin.formats = formats;
