@@ -20,8 +20,8 @@
 #include "config.h"
 #endif
 
+#include "ogmrip-container.h"
 #include "ogmrip-fs.h"
-#include "ogmrip-mp4.h"
 #include "ogmrip-mplayer.h"
 #include "ogmrip-plugin.h"
 #include "ogmrip-version.h"
@@ -34,13 +34,22 @@
 #include <glib/gi18n-lib.h>
 #include <glib/gstdio.h>
 
-#define OGMRIP_MP4_GET_PRIVATE(o) \
-  (G_TYPE_INSTANCE_GET_PRIVATE ((o), OGMRIP_TYPE_MP4, OGMRipMp4Priv))
-
 #define PROGRAM "MP4Box"
 
-struct _OGMRipMp4Priv
+#define OGMRIP_TYPE_MP4           (ogmrip_mp4_get_type ())
+#define OGMRIP_MP4(obj)           (G_TYPE_CHECK_INSTANCE_CAST ((obj), OGMRIP_TYPE_MP4, OGMRipMp4))
+#define OGMRIP_MP4_CLASS(klass)   (G_TYPE_CHECK_CLASS_CAST ((klass), OGMRIP_TYPE_MP4, OGMRipMp4Class))
+#define OGMRIP_IS_MP4(obj)        (G_TYPE_CHECK_INSTANCE_TYPE ((obj), OGMRIP_TYPE_MP4))
+#define OGMRIP_IS_MP4_CLASS(obj)  (G_TYPE_CHECK_CLASS_TYPE ((klass), OGMRIP_TYPE_MP4))
+#define OGMRIP_MP4_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), OGMRIP_TYPE_MP4, OGMRipMp4Class))
+
+typedef struct _OGMRipMp4      OGMRipMp4;
+typedef struct _OGMRipMp4Class OGMRipMp4Class;
+
+struct _OGMRipMp4
 {
+  OGMRipContainer parent_instance;
+
   guint nstreams;
   guint streams;
   guint old_percent;
@@ -50,6 +59,12 @@ struct _OGMRipMp4Priv
   guint split_percent;
 };
 
+struct _OGMRipMp4Class
+{
+  OGMRipContainerClass parent_class;
+};
+
+GType ogmrip_mp4_get_type (void);
 static gint ogmrip_mp4_run (OGMJobSpawn *spawn);
 
 static void
@@ -73,6 +88,10 @@ ogmrip_mp4_append_audio_file (OGMRipContainer *mp4, const gchar *filename,
       case OGMRIP_FORMAT_VORBIS:
         fmt = "ogg";
         break;
+      case OGMRIP_FORMAT_AC3:
+      case OGMRIP_FORMAT_COPY:
+        fmt = "ac3";
+        break;
       default:
         fmt = NULL;
         break;
@@ -86,9 +105,9 @@ ogmrip_mp4_append_audio_file (OGMRipContainer *mp4, const gchar *filename,
       if (language > -1)
         iso639_2 = ogmdvd_get_language_iso639_2 (language);
       if (iso639_2)
-        g_ptr_array_add (argv, g_strdup_printf ("%s:fmt=%s:lang=%s:#audio", filename, fmt, iso639_2));
+        g_ptr_array_add (argv, g_strdup_printf ("%s:fmt=%s:lang=%s:group=1:#audio", filename, fmt, iso639_2));
       else
-        g_ptr_array_add (argv, g_strdup_printf ("%s:fmt=%s:#audio", filename, fmt));
+        g_ptr_array_add (argv, g_strdup_printf ("%s:fmt=%s:group=1:#audio", filename, fmt));
     }
   }
 }
@@ -218,7 +237,38 @@ ogmrip_mp4_get_output_fps (OGMRipCodec *codec)
 }
 
 static gchar **
-ogmrip_mp4_extract_command (OGMRipVideoCodec *video, const gchar *output)
+ogmrip_mp4box_extract_command (OGMRipVideoCodec *video)
+{
+  GPtrArray *argv;
+  const gchar *filename;
+
+  argv = g_ptr_array_new ();
+  g_ptr_array_add (argv, g_strdup (PROGRAM));
+  g_ptr_array_add (argv, g_strdup ("-aviraw"));
+  g_ptr_array_add (argv, g_strdup ("video"));
+
+  filename = ogmrip_codec_get_output (OGMRIP_CODEC (video));
+  g_ptr_array_add (argv, g_strdup (filename));
+
+  g_ptr_array_add (argv, NULL);
+
+  return (gchar **) g_ptr_array_free (argv, FALSE);
+}
+
+static gdouble
+ogmrip_mp4box_extract_watch (OGMJobExec *exec, const gchar *buffer, OGMRipContainer *mp4)
+{
+  gchar *sep;
+  guint percent;
+
+  if ((sep = strrchr (buffer, '(')) && sscanf (sep, "(%u/100)", &percent) == 1)
+    return percent / 100.0;
+
+  return -1.0;
+}
+
+static gchar **
+ogmrip_mencoder_extract_command (OGMRipVideoCodec *video, const gchar *output)
 {
   GPtrArray *argv;
   const gchar *filename;
@@ -275,44 +325,65 @@ ogmrip_mp4_get_n_audio_files (OGMRipContainer *mp4)
   return n_audio;
 }
 
+static gint
+ogmrip_mp4_get_n_subp_files (OGMRipContainer *mp4)
+{
+  GSList *files, *file;
+  guint n_subp = 0;
+
+  files = ogmrip_container_get_files (mp4);
+  for (file = files; file; file = file->next)
+  {
+    if (ogmrip_file_get_type (OGMRIP_FILE (file->data)) == OGMRIP_FILE_TYPE_SUBP)
+      n_subp ++;
+  }
+
+  g_slist_free (files);
+
+  return n_subp;
+}
+
 static gchar **
 ogmrip_mp4_create_command (OGMRipContainer *mp4, const gchar *input, const gchar *output)
 {
   GPtrArray *argv;
   OGMRipVideoCodec *video;
-  const gchar *fmt, *label;
+  const gchar *label, *fmt = NULL;
   gchar fps[8];
-  gint format;
 
-  video = ogmrip_container_get_video (mp4);
-
-  format = ogmrip_plugin_get_video_codec_format (G_TYPE_FROM_INSTANCE (video));
-  switch (format)
+  if ((video = ogmrip_container_get_video (mp4)))
   {
-    case OGMRIP_FORMAT_MPEG4:
-      fmt = "mpeg4-video";
-      break;
-    case OGMRIP_FORMAT_MPEG2:
-      fmt = "mpeg2-video";
-      break;
-    case OGMRIP_FORMAT_H264:
-      fmt = "h264";
-      break;
-    case OGMRIP_FORMAT_THEORA:
-      fmt = "ogg";
-      break;
-    default:
-      fmt = NULL;
-      break;
-  }
+    gint format;
 
-  if (!fmt)
-    return NULL;
+    format = ogmrip_plugin_get_video_codec_format (G_TYPE_FROM_INSTANCE (video));
+    switch (format)
+    {
+      case OGMRIP_FORMAT_MPEG4:
+        fmt = "mpeg4-video";
+        break;
+      case OGMRIP_FORMAT_MPEG2:
+        fmt = "mpeg2-video";
+        break;
+      case OGMRIP_FORMAT_H264:
+        fmt = "h264";
+        break;
+      case OGMRIP_FORMAT_THEORA:
+        fmt = "ogg";
+        break;
+      default:
+        fmt = NULL;
+        break;
+    }
+
+    if (!fmt)
+      return NULL;
+  }
 
   argv = g_ptr_array_new ();
   g_ptr_array_add (argv, g_strdup (PROGRAM));
 
-  if (ogmrip_container_get_n_audio (mp4) + ogmrip_mp4_get_n_audio_files (mp4) <= 1)
+  if (ogmrip_container_get_n_audio (mp4) + ogmrip_mp4_get_n_audio_files (mp4) <= 1 &&
+      ogmrip_container_get_n_subp (mp4) + ogmrip_mp4_get_n_subp_files (mp4) < 1)
     g_ptr_array_add (argv, g_strdup ("-isma"));
 
   g_ptr_array_add (argv, g_strdup ("-nodrop"));
@@ -331,14 +402,17 @@ ogmrip_mp4_create_command (OGMRipContainer *mp4, const gchar *input, const gchar
     g_ptr_array_add (argv, g_strdup_printf ("name=%s", label));
   }
 
-  if (!input)
-    input = ogmrip_codec_get_output (OGMRIP_CODEC (video));
+  if (fmt)
+  {
+    if (!input)
+      input = ogmrip_codec_get_output (OGMRIP_CODEC (video));
 
-  g_ascii_formatd (fps, 8, "%.3f",
-      ogmrip_mp4_get_output_fps (OGMRIP_CODEC (video)));
+    g_ascii_formatd (fps, 8, "%.3f",
+        ogmrip_mp4_get_output_fps (OGMRIP_CODEC (video)));
 
-  g_ptr_array_add (argv, g_strdup ("-add"));
-  g_ptr_array_add (argv, g_strdup_printf ("%s:fmt=%s:fps=%s#video", input, fmt, fps));
+    g_ptr_array_add (argv, g_strdup ("-add"));
+    g_ptr_array_add (argv, g_strdup_printf ("%s:fmt=%s:fps=%s#video", input, fmt, fps));
+  }
 
   ogmrip_container_foreach_audio (mp4, 
       (OGMRipContainerCodecFunc) ogmrip_mp4_foreach_audio, argv);
@@ -364,12 +438,12 @@ ogmrip_mp4_create_watch (OGMJobExec *exec, const gchar *buffer, OGMRipMp4 *mp4)
 
   if ((sep = strrchr (buffer, '(')) && sscanf (sep, "(%u/100)", &percent) == 1)
   {
-    if (percent < mp4->priv->old_percent)
-      mp4->priv->streams ++;
+    if (percent < mp4->old_percent)
+      mp4->streams ++;
 
-    mp4->priv->old_percent = percent;
+    mp4->old_percent = percent;
 
-    return mp4->priv->streams / (gdouble) mp4->priv->nstreams + percent / (mp4->priv->nstreams * 100.0);
+    return mp4->streams / (gdouble) mp4->nstreams + percent / (mp4->nstreams * 100.0);
   }
 
   return -1.0;
@@ -402,22 +476,22 @@ static gdouble
 ogmrip_mp4_split_watch (OGMJobExec *exec, const gchar *buffer, OGMRipMp4 *mp4)
 {
   gchar *sep;
-  gint percent;
+  guint percent;
 
   if ((sep = strrchr (buffer, '(')) && sscanf (sep, "(%u/100)", &percent) == 1)
   {
     if (g_str_has_prefix (buffer, "Splitting:"))
     {
-      mp4->priv->split_percent = percent;
+      mp4->split_percent = percent;
 
-      return (percent + 100 * mp4->priv->splits) / (100.0 * (mp4->priv->nsplits + 1));
+      return (percent + 100 * mp4->splits) / (100.0 * (mp4->nsplits + 1));
     }
     else if (g_str_has_prefix (buffer, "ISO File Writing:"))
     {
-      if (percent < mp4->priv->split_percent)
-        mp4->priv->splits ++;
+      if (percent < mp4->split_percent)
+        mp4->splits ++;
 
-      return (percent + mp4->priv->split_percent + 100 * mp4->priv->splits) / (100.0 * (mp4->priv->nsplits + 1));
+      return (percent + mp4->split_percent + 100 * mp4->splits) / (100.0 * (mp4->nsplits + 1));
     }
   }
 
@@ -436,14 +510,27 @@ ogmrip_mp4_class_init (OGMRipMp4Class *klass)
   container_class = OGMRIP_CONTAINER_CLASS (klass);
 
   spawn_class->run = ogmrip_mp4_run;
-
-  g_type_class_add_private (klass, sizeof (OGMRipMp4Priv));
 }
 
 static void
 ogmrip_mp4_init (OGMRipMp4 *mp4)
 {
-  mp4->priv = OGMRIP_MP4_GET_PRIVATE (mp4);
+}
+
+static gchar *
+ogmrip_mp4_get_h264_filename (OGMRipVideoCodec *video)
+{
+  const gchar *name;
+  gchar *dot, *filename;
+
+  name = ogmrip_codec_get_output (OGMRIP_CODEC (video));
+  dot = strrchr (name, '.');
+
+  filename = g_new0 (gchar, dot - name + 12);
+  strncpy (filename, name, dot - name);
+  strcat (filename, "_video.h264");
+
+  return filename;
 }
 
 static void
@@ -470,7 +557,7 @@ ogmrip_mp4_run (OGMJobSpawn *spawn)
   mp4 = OGMRIP_MP4 (spawn);
 
   output = ogmrip_container_get_output (OGMRIP_CONTAINER (spawn));
-  ogmrip_container_get_split (OGMRIP_CONTAINER (spawn), &mp4->priv->nsplits, NULL);
+  ogmrip_container_get_split (OGMRIP_CONTAINER (spawn), &mp4->nsplits, NULL);
 
   queue = ogmjob_queue_new ();
   ogmjob_container_add (OGMJOB_CONTAINER (spawn), queue);
@@ -479,14 +566,37 @@ ogmrip_mp4_run (OGMJobSpawn *spawn)
   video = ogmrip_container_get_video (OGMRIP_CONTAINER (spawn));
   if (ogmrip_plugin_get_video_codec_format (G_TYPE_FROM_INSTANCE (video)) == OGMRIP_FORMAT_H264)
   {
-    filename = ogmrip_fs_mktemp ("video.XXXXXX", NULL);
+    gboolean global_header = FALSE;
 
-    argv = ogmrip_mp4_extract_command (video, filename);
-    if (!argv)
-      return OGMJOB_RESULT_ERROR;
+    if (g_object_class_find_property (G_OBJECT_GET_CLASS (video), "global_header"))
+      g_object_get (video, "global_header", &global_header, NULL);
+
+    if (global_header)
+    {
+      filename = ogmrip_fs_mktemp ("video.XXXXXX", NULL);
+
+      argv = ogmrip_mencoder_extract_command (video, filename);
+      if (!argv)
+      {
+        g_free (filename);
+        return OGMJOB_RESULT_ERROR;
+      }
+
+      child = ogmjob_exec_newv (argv);
+      ogmjob_exec_add_watch_full (OGMJOB_EXEC (child), (OGMJobWatch) ogmrip_mencoder_container_watch, spawn, TRUE, FALSE, FALSE);
+    }
+    else
+    {
+      argv = ogmrip_mp4box_extract_command (video);
+      if (!argv)
+        return OGMJOB_RESULT_ERROR;
+
+      child = ogmjob_exec_newv (argv);
+      ogmjob_exec_add_watch_full (OGMJOB_EXEC (child), (OGMJobWatch) ogmrip_mp4box_extract_watch, spawn, TRUE, FALSE, FALSE);
+
+      filename = ogmrip_mp4_get_h264_filename (video);
+    }
 
-    child = ogmjob_exec_newv (argv);
-    ogmjob_exec_add_watch_full (OGMJOB_EXEC (child), (OGMJobWatch) ogmrip_mencoder_container_watch, spawn, TRUE, FALSE, FALSE);
     ogmjob_container_add (OGMJOB_CONTAINER (queue), child);
     g_object_unref (child);
   }
@@ -498,22 +608,22 @@ ogmrip_mp4_run (OGMJobSpawn *spawn)
 
     ogmrip_container_foreach_subp (OGMRIP_CONTAINER (spawn), (OGMRipContainerCodecFunc) ogmrip_mp4_get_n_vobsub, &nvobsub);
 
-    mp4->priv->old_percent = 0;
-    mp4->priv->nstreams = 2 + ogmrip_container_get_n_audio (OGMRIP_CONTAINER (spawn)) + nvobsub;
-    mp4->priv->streams = 0;
+    mp4->old_percent = 0;
+    mp4->nstreams = 2 + ogmrip_container_get_n_audio (OGMRIP_CONTAINER (spawn)) + nvobsub;
+    mp4->streams = 0;
 
     child = ogmjob_exec_newv (argv);
     ogmjob_exec_add_watch_full (OGMJOB_EXEC (child), (OGMJobWatch) ogmrip_mp4_create_watch, spawn, TRUE, FALSE, FALSE);
     ogmjob_container_add (OGMJOB_CONTAINER (queue), child);
     g_object_unref (child);
 
-    if (mp4->priv->nsplits > 1 && result == OGMJOB_RESULT_COMPLETED)
+    if (mp4->nsplits > 1 && result == OGMJOB_RESULT_SUCCESS)
     {
       argv = ogmrip_mp4_split_command (OGMRIP_CONTAINER (spawn), output);
       if (argv)
       {
-        mp4->priv->split_percent = 0;
-        mp4->priv->splits = 0;
+        mp4->split_percent = 0;
+        mp4->splits = 0;
 
         child = ogmjob_exec_newv (argv);
         ogmjob_exec_add_watch_full (OGMJOB_EXEC (child), (OGMJobWatch) ogmrip_mp4_split_watch, spawn, TRUE, FALSE, FALSE);
@@ -530,34 +640,19 @@ ogmrip_mp4_run (OGMJobSpawn *spawn)
   if (filename)
     ogmrip_fs_unref (filename, TRUE);
 
-  if (mp4->priv->nsplits > 1)
+  if (mp4->nsplits > 1)
     ogmrip_fs_unref (g_strdup (output), TRUE);
 
   return result;
 }
 
-/**
- * ogmrip_mp4_new:
- * @output: The output file
- *
- * Creates a new #OGMRipMp4.
- *
- * Returns: The new #OGMRipMp4
- */
-OGMJobSpawn *
-ogmrip_mp4_new (const gchar *output)
-{
-  g_return_val_if_fail (output && *output, NULL);
-
-  return g_object_new (OGMRIP_TYPE_MP4, "output", output, NULL);
-}
-
 static OGMRipContainerPlugin mp4_plugin =
 {
   NULL,
   G_TYPE_NONE,
   "mp4",
   N_("Mpeg-4 Media (MP4)"),
+  FALSE,
   TRUE,
   G_MAXINT,
   G_MAXINT,
@@ -575,18 +670,38 @@ static gint formats[] =
   OGMRIP_FORMAT_VORBIS,
   OGMRIP_FORMAT_SRT,
   OGMRIP_FORMAT_VOBSUB,
+  -1,
+  -1,
   -1
 };
 
 OGMRipContainerPlugin *
-ogmrip_init_plugin (void)
+ogmrip_init_plugin (GError **error)
 {
-  gchar *fullname;
+  gchar *output;
+  gboolean match;
 
-  fullname = g_find_program_in_path (PROGRAM);
-  if (!fullname)
+  g_return_val_if_fail (error == NULL || *error == NULL, NULL);
+
+  if (!g_spawn_command_line_sync (PROGRAM " -version", &output, NULL, NULL, NULL))
+  {
+    g_set_error (error, OGMRIP_PLUGIN_ERROR, OGMRIP_PLUGIN_ERROR_REQ, _("MP4Box is missing"));
     return NULL;
-  g_free (fullname);
+  }
+
+  match = g_str_has_prefix (output, "MP4Box - GPAC version 0.4.5");
+  g_free (output);
+
+  if (match)
+  {
+    guint i = 0;
+
+    while (formats[i] != -1)
+      i++;
+
+    formats[i] = OGMRIP_FORMAT_AC3;
+    formats[i+1] = OGMRIP_FORMAT_COPY;
+  }
 
   mp4_plugin.type = OGMRIP_TYPE_MP4;
   mp4_plugin.formats = formats;
