@@ -36,7 +36,6 @@
 
 #include <glib/gstdio.h>
 
-// static const gchar *deinterlacer[] = { "lb", "li", "ci", "md", "fd", "l5", "kerndeint", "yadif=3,mcdeint=2:1:10" };
 static const gchar *deinterlacer[] = { "lb", "li", "ci", "md", "fd", "l5", "kerndeint", "yadif=0" };
 
 static gint
@@ -65,6 +64,18 @@ ogmrip_mplayer_map_audio_id (OGMDvdAudioStream *astream)
   return aid;
 }
 
+glong
+ogmrip_mplayer_get_frames (OGMRipCodec *codec)
+{
+  gdouble length;
+  guint num, denom;
+
+  length = ogmrip_codec_get_length (codec, NULL);
+  ogmrip_codec_get_framerate (codec, &num, &denom);
+
+  return length / (gdouble) denom * num;
+}
+
 static gchar *
 ogmrip_mplayer_get_output_fps (OGMRipCodec *codec, OGMDvdTitle *title)
 {
@@ -228,6 +239,7 @@ ogmrip_mplayer_wav_command (OGMRipAudioCodec *audio, gboolean header, const gcha
   GPtrArray *argv;
   GString *options;
 
+  gdouble start, length;
   const gchar *device;
   gchar *chap, *ofps;
   gint vid;
@@ -257,16 +269,21 @@ ogmrip_mplayer_wav_command (OGMRipAudioCodec *audio, gboolean header, const gcha
   }
 
   ofps = ogmrip_mplayer_get_output_fps (OGMRIP_CODEC (audio), title);
-  if (ofps)
-    g_free (ofps);
-  else
+  if (!ofps)
   {
     g_ptr_array_add (argv, g_strdup ("-mc"));
     g_ptr_array_add (argv, g_strdup ("0"));
   }
+  g_free (ofps);
+
+  length = ogmrip_codec_get_play_length (OGMRIP_CODEC (audio));
+
+  if (length <= 0.0)
+  {
+    g_ptr_array_add (argv, g_strdup ("-vc"));
+    g_ptr_array_add (argv, g_strdup ("dummy"));
+  }
 
-  g_ptr_array_add (argv, g_strdup ("-vc"));
-  g_ptr_array_add (argv, g_strdup ("dummy"));
   g_ptr_array_add (argv, g_strdup ("-vo"));
   g_ptr_array_add (argv, g_strdup ("null"));
   g_ptr_array_add (argv, g_strdup ("-ao"));
@@ -353,6 +370,22 @@ ogmrip_mplayer_wav_command (OGMRipAudioCodec *audio, gboolean header, const gcha
     g_ptr_array_add (argv, chap);
   }
 
+  start = ogmrip_codec_get_start (OGMRIP_CODEC (audio));
+  if (start > 0.0)
+  {
+    g_ptr_array_add (argv, g_strdup ("-ss"));
+    g_ptr_array_add (argv, g_strdup_printf ("%.0lf", start));
+  }
+
+  if (length > 0.0)
+  {
+    guint num, denom;
+
+    ogmrip_codec_get_framerate (OGMRIP_CODEC (audio), &num, &denom);
+    g_ptr_array_add (argv, g_strdup ("-frames"));
+    g_ptr_array_add (argv, g_strdup_printf ("%.0lf", length * num / denom));
+  }
+
   ogmrip_mplayer_append_edl (OGMRIP_CODEC (audio), argv, FALSE);
 
   g_ptr_array_add (argv, g_strdup ("-aid"));
@@ -424,6 +457,7 @@ ogmrip_mencoder_audio_command (OGMRipAudioCodec *audio, const gchar *output)
 {
   GPtrArray *argv;
   const gchar *device;
+  gdouble start, length;
   gchar *ofps, *chap;
   gint vid;
 
@@ -464,6 +498,23 @@ ogmrip_mencoder_audio_command (OGMRipAudioCodec *audio, const gchar *output)
     g_ptr_array_add (argv, chap);
   }
 
+  start = ogmrip_codec_get_start (OGMRIP_CODEC (audio));
+  if (start > 0.0)
+  {
+    g_ptr_array_add (argv, g_strdup ("-ss"));
+    g_ptr_array_add (argv, g_strdup_printf ("%.0lf", start));
+  }
+
+  length = ogmrip_codec_get_play_length (OGMRIP_CODEC (audio));
+  if (length > 0.0)
+  {
+    guint num, denom;
+
+    ogmrip_codec_get_framerate (OGMRIP_CODEC (audio), &num, &denom);
+    g_ptr_array_add (argv, g_strdup ("-frames"));
+    g_ptr_array_add (argv, g_strdup_printf ("%.0lf", length * num / denom));
+  }
+
   ogmrip_mplayer_append_edl (OGMRIP_CODEC (audio), argv, TRUE);
 
   g_ptr_array_add (argv, g_strdup ("-aid"));
@@ -501,13 +552,15 @@ ogmrip_mencoder_video_command (OGMRipVideoCodec *video, const gchar *output, gui
   OGMRipScalerType scaler;
   OGMRipDeintType deint;
   OGMDvdAudioStream *astream;
+  OGMDvdSubpStream *sstream;
   GString *options, *pp;
 
   const gchar *device;
   guint max_width, max_height;
   guint crop_x, crop_y, crop_width, crop_height;
   guint scale_width, scale_height;
-  gboolean crop, scale, expand;
+  gboolean crop, scale, expand, forced;
+  gdouble start, length;
   gchar *ofps, *chap;
 
   title = ogmrip_codec_get_input (OGMRIP_CODEC (video));
@@ -543,6 +596,19 @@ ogmrip_mencoder_video_command (OGMRipVideoCodec *video, const gchar *output, gui
   else
     g_ptr_array_add (argv, g_strdup ("-nosound"));
 
+  sstream = ogmrip_video_codec_get_hard_subp (video, &forced);
+  if (sstream)
+  {
+    g_ptr_array_add (argv, g_strdup ("-spuaa"));
+    g_ptr_array_add (argv, g_strdup ("20"));
+    g_ptr_array_add (argv, g_strdup ("-sid"));
+    g_ptr_array_add (argv, g_strdup_printf ("%d",
+          ogmdvd_stream_get_id (OGMDVD_STREAM (sstream))));
+
+    if (forced)
+      g_ptr_array_add (argv, g_strdup ("-forcedsubsonly"));
+  }
+
   scaler = ogmrip_video_codec_get_scaler (video);
   g_ptr_array_add (argv, g_strdup ("-sws"));
   g_ptr_array_add (argv, g_strdup_printf ("%d", MAX (scaler, 0)));
@@ -557,6 +623,23 @@ ogmrip_mencoder_video_command (OGMRipVideoCodec *video, const gchar *output, gui
     g_ptr_array_add (argv, chap);
   }
 
+  start = ogmrip_codec_get_start (OGMRIP_CODEC (video));
+  if (start > 0.0)
+  {
+    g_ptr_array_add (argv, g_strdup ("-ss"));
+    g_ptr_array_add (argv, g_strdup_printf ("%.0lf", start));
+  }
+
+  length = ogmrip_codec_get_play_length (OGMRIP_CODEC (video));
+  if (length > 0.0)
+  {
+    guint num, denom;
+
+    ogmrip_codec_get_framerate (OGMRIP_CODEC (video), &num, &denom);
+    g_ptr_array_add (argv, g_strdup ("-frames"));
+    g_ptr_array_add (argv, g_strdup_printf ("%.0lf", length * num / denom));
+  }
+
   ogmrip_mplayer_append_edl (OGMRIP_CODEC (video), argv, TRUE);
 
   options = g_string_new (NULL);
@@ -692,7 +775,7 @@ ogmrip_mplayer_video_watch (OGMJobExec *exec, const gchar *buffer, OGMRipVideoCo
   gint frame, decoded;
 
   if (sscanf (buffer, "V:%s %d/%d", v, &frame, &decoded) == 3)
-    return decoded / (gdouble) ogmrip_codec_get_frames (OGMRIP_CODEC (video));
+    return decoded / (gdouble) ogmrip_mplayer_get_frames (OGMRIP_CODEC (video));
 
   return -1.0;
 }
@@ -717,6 +800,7 @@ ogmrip_mplayer_video_command (OGMRipVideoCodec *video, const gchar *output)
   GPtrArray *argv;
 
   const gchar *device;
+  gdouble start, length;
   guint max_width, max_height;
   guint crop_x, crop_y, crop_width, crop_height;
   guint scale_width, scale_height;
@@ -844,16 +928,12 @@ ogmrip_mplayer_video_command (OGMRipVideoCodec *video, const gchar *output)
   }
 
   ofps = ogmrip_mplayer_get_output_fps (OGMRIP_CODEC (video), title);
-  if (ofps)
-  {
-    g_ptr_array_add (argv, g_strdup ("-ofps"));
-    g_ptr_array_add (argv, ofps);
-  }
-  else
+  if (!ofps)
   {
     g_ptr_array_add (argv, g_strdup ("-mc"));
     g_ptr_array_add (argv, g_strdup ("0"));
   }
+  g_free (ofps);
 
   g_ptr_array_add (argv, g_strdup ("-dvdangle"));
   g_ptr_array_add (argv, g_strdup_printf ("%d", ogmrip_video_codec_get_angle (video)));
@@ -865,6 +945,23 @@ ogmrip_mplayer_video_command (OGMRipVideoCodec *video, const gchar *output)
     g_ptr_array_add (argv, chap);
   }
 
+  start = ogmrip_codec_get_start (OGMRIP_CODEC (video));
+  if (start > 0.0)
+  {
+    g_ptr_array_add (argv, g_strdup ("-ss"));
+    g_ptr_array_add (argv, g_strdup_printf ("%.0lf", start));
+  }
+
+  length = ogmrip_codec_get_play_length (OGMRIP_CODEC (video));
+  if (length > 0.0)
+  {
+    guint num, denom;
+
+    ogmrip_codec_get_framerate (OGMRIP_CODEC (video), &num, &denom);
+    g_ptr_array_add (argv, g_strdup ("-frames"));
+    g_ptr_array_add (argv, g_strdup_printf ("%.0lf", length * num / denom));
+  }
+
   ogmrip_mplayer_append_edl (OGMRIP_CODEC (video), argv, FALSE);
 
   device = ogmdvd_disc_get_device (ogmdvd_title_get_disc (title));
@@ -918,6 +1015,7 @@ ogmrip_mencoder_vobsub_command (OGMRipSubpCodec *subp, const gchar *output)
   OGMDvdSubpStream *sstream;
 
   GPtrArray *argv;
+  gdouble start, length;
   const gchar *device;
   gchar *ofps, *chap;
   gint vid, sid;
@@ -982,6 +1080,23 @@ ogmrip_mencoder_vobsub_command (OGMRipSubpCodec *subp, const gchar *output)
     g_ptr_array_add (argv, chap);
   }
 
+  start = ogmrip_codec_get_start (OGMRIP_CODEC (subp));
+  if (start > 0.0)
+  {
+    g_ptr_array_add (argv, g_strdup ("-ss"));
+    g_ptr_array_add (argv, g_strdup_printf ("%.0lf", start));
+  }
+
+  length = ogmrip_codec_get_play_length (OGMRIP_CODEC (subp));
+  if (length > 0.0)
+  {
+    guint num, denom;
+
+    ogmrip_codec_get_framerate (OGMRIP_CODEC (subp), &num, &denom);
+    g_ptr_array_add (argv, g_strdup ("-frames"));
+    g_ptr_array_add (argv, g_strdup_printf ("%.0lf", length * num / denom));
+  }
+
   ogmrip_mplayer_append_edl (OGMRIP_CODEC (subp), argv, TRUE);
 
   device = ogmdvd_disc_get_device (ogmdvd_title_get_disc (title));
