@@ -16,6 +16,10 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 /**
  * SECTION:ogmrip-plugin
  * @title: Plugins System
@@ -29,10 +33,15 @@
 #include "ogmrip-video-codec.h"
 #include "ogmrip-subp-codec.h"
 
+#include "ogmrip-hardsub.h"
+#include "ogmrip-novideo.h"
+
 #include "ogmjob-log.h"
 
 #include <string.h>
 
+#include <glib/gi18n-lib.h>
+
 typedef struct _OGMRipPlugin OGMRipPlugin;
 
 struct _OGMRipPlugin
@@ -54,13 +63,24 @@ struct _OGMRipPluginCodec
   gint format;
 };
 
-typedef OGMRipPlugin * (* OGMRipPluginInit) (void);
+typedef OGMRipPlugin * (* OGMRipPluginInit) (GError **error);
 
 static GSList *video_plugins = NULL;
 static GSList *audio_plugins = NULL;
 static GSList *subp_plugins  = NULL;
 static GSList *container_plugins   = NULL;
 
+GQuark
+ogmrip_plugin_error_quark (void)
+{
+  static GQuark quark = 0;
+
+  if (quark == 0)
+    quark = g_quark_from_static_string ("ogmrip-plugin-error-quark");
+
+  return quark;
+}
+
 static gint
 ogmrip_plugin_compare (OGMRipPlugin *plugin1, OGMRipPlugin *plugin2)
 {
@@ -68,12 +88,11 @@ ogmrip_plugin_compare (OGMRipPlugin *plugin1, OGMRipPlugin *plugin2)
 }
 
 static GSList *
-ogmrip_plugin_load (const gchar *dirname, GType type)
+ogmrip_plugin_load (GSList *slist, const gchar *dirname, GType type)
 {
-  GSList *slist = NULL;
-
   GModule *module;
   GPatternSpec *pspec;
+  GError *error;
   GDir *dir;
 
   OGMRipPlugin *plugin;
@@ -94,6 +113,7 @@ ogmrip_plugin_load (const gchar *dirname, GType type)
     while ((filename = g_dir_read_name (dir)))
     {
       init = NULL;
+      error = NULL;
 
       if (!g_pattern_match_string (pspec, filename))
         continue;
@@ -124,10 +144,21 @@ ogmrip_plugin_load (const gchar *dirname, GType type)
         continue;
       }
 
-      plugin = (* init) ();
+      plugin = (* init) (&error);
       if (!plugin)
       {
-        g_print ("Plugin %s disabled, some requirements are not available\n", filename);
+        gchar *msg;
+
+        msg = g_strdup_printf (_("Plugin %s disabled"), filename);
+        if (!error)
+          g_print ("%s: %s\n", msg, _("some requirements are not available"));
+        else
+        {
+          g_print ("%s: %s\n", msg, error->message);
+          g_error_free (error);
+        }
+        g_free (msg);
+
         g_module_close (module);
         continue;
       }
@@ -150,6 +181,15 @@ ogmrip_plugin_load (const gchar *dirname, GType type)
   return slist;
 }
 
+#define OGMRIP_VIDEO_PLUGINS_DIR \
+  OGMRIP_LIB_DIR G_DIR_SEPARATOR_S "ogmrip" G_DIR_SEPARATOR_S "video-plugins"
+#define OGMRIP_AUDIO_PLUGINS_DIR \
+  OGMRIP_LIB_DIR G_DIR_SEPARATOR_S "ogmrip" G_DIR_SEPARATOR_S "audio-plugins"
+#define OGMRIP_SUBP_PLUGINS_DIR \
+  OGMRIP_LIB_DIR G_DIR_SEPARATOR_S "ogmrip" G_DIR_SEPARATOR_S "subp-plugins"
+#define OGMRIP_CONTAINER_PLUGINS_DIR \
+  OGMRIP_LIB_DIR G_DIR_SEPARATOR_S "ogmrip" G_DIR_SEPARATOR_S "container-plugins"
+
 /**
  * ogmrip_plugin_init:
  *
@@ -158,23 +198,62 @@ ogmrip_plugin_load (const gchar *dirname, GType type)
 void
 ogmrip_plugin_init (void)
 {
+  gchar *dir;
+
   if (!video_plugins)
-    video_plugins = ogmrip_plugin_load (OGMRIP_LIB_DIR "/ogmrip/video-plugins", OGMRIP_TYPE_VIDEO_CODEC);
+  {
+    OGMRipVideoPlugin *plugin;
+
+    video_plugins = ogmrip_plugin_load (video_plugins, OGMRIP_VIDEO_PLUGINS_DIR, OGMRIP_TYPE_VIDEO_CODEC);
+
+    dir = g_build_filename (g_get_home_dir (), ".ogmrip", "video-plugins", NULL);
+    video_plugins = ogmrip_plugin_load (video_plugins, dir, OGMRIP_TYPE_VIDEO_CODEC);
+    g_free (dir);
+
+    plugin = ogmrip_novideo_get_plugin ();
+    if (plugin)
+      video_plugins = g_slist_insert_sorted (video_plugins, plugin, (GCompareFunc) ogmrip_plugin_compare);
+  }
 
   if (!audio_plugins)
-    audio_plugins = ogmrip_plugin_load (OGMRIP_LIB_DIR "/ogmrip/audio-plugins", OGMRIP_TYPE_AUDIO_CODEC);
+  {
+    audio_plugins = ogmrip_plugin_load (audio_plugins, OGMRIP_AUDIO_PLUGINS_DIR, OGMRIP_TYPE_AUDIO_CODEC);
+
+    dir = g_build_filename (g_get_home_dir (), ".ogmrip", "audio-plugins", NULL);
+    audio_plugins = ogmrip_plugin_load (audio_plugins, dir, OGMRIP_TYPE_AUDIO_CODEC);
+    g_free (dir);
+  }
 
   if (!subp_plugins)
-    subp_plugins = ogmrip_plugin_load (OGMRIP_LIB_DIR "/ogmrip/subp-plugins", OGMRIP_TYPE_SUBP_CODEC);
+  {
+    OGMRipSubpPlugin *plugin;
+
+    subp_plugins = ogmrip_plugin_load (subp_plugins, OGMRIP_SUBP_PLUGINS_DIR, OGMRIP_TYPE_SUBP_CODEC);
+
+    dir = g_build_filename (g_get_home_dir (), ".ogmrip", "subp-plugins", NULL);
+    subp_plugins = ogmrip_plugin_load (subp_plugins, dir, OGMRIP_TYPE_SUBP_CODEC);
+    g_free (dir);
+
+    plugin = ogmrip_hardsub_get_plugin ();
+    if (plugin)
+      subp_plugins = g_slist_insert_sorted (subp_plugins, plugin, (GCompareFunc) ogmrip_plugin_compare);
+  }
 
   if (!container_plugins)
-    container_plugins = ogmrip_plugin_load (OGMRIP_LIB_DIR "/ogmrip/container-plugins", OGMRIP_TYPE_CONTAINER);
+  {
+    container_plugins = ogmrip_plugin_load (container_plugins, OGMRIP_CONTAINER_PLUGINS_DIR, OGMRIP_TYPE_CONTAINER);
+
+    dir = g_build_filename (g_get_home_dir (), ".ogmrip", "container-plugins", NULL);
+    container_plugins = ogmrip_plugin_load (container_plugins, dir, OGMRIP_TYPE_CONTAINER);
+    g_free (dir);
+  }
 }
 
 static void
 ogmrip_plugin_close_module (OGMRipPlugin *plugin)
 {
-  g_module_close (plugin->module);
+  if (plugin->module)
+    g_module_close (plugin->module);
 }
 
 /**
@@ -499,6 +578,72 @@ ogmrip_plugin_get_container_bframes (GType container)
   return FALSE;
 }
 
+/**
+ * ogmrip_plugin_get_container_max_audio:
+ * @container: A container type
+ *
+ * Returns the number of audio streams the given container can contain.
+ *
+ * Returns: the number of audio streams, or -1
+ */
+gint
+ogmrip_plugin_get_container_max_audio (GType container)
+{
+  OGMRipContainerPlugin *plugin;
+
+  g_return_val_if_fail (g_type_is_a (container, OGMRIP_TYPE_CONTAINER), -1);
+
+  plugin = ogmrip_plugin_find_container_by_type (container_plugins, container);
+  if (!plugin)
+    return -1;
+
+  return plugin->max_audio;
+}
+
+/**
+ * ogmrip_plugin_get_container_max_subp:
+ * @container: A container type
+ *
+ * Returns the number of subtitle streams the given container can contain.
+ *
+ * Returns: the number of subtitle streams, or -1
+ */
+gint
+ogmrip_plugin_get_container_max_subp (GType container)
+{
+  OGMRipContainerPlugin *plugin;
+
+  g_return_val_if_fail (g_type_is_a (container, OGMRIP_TYPE_CONTAINER), -1);
+
+  plugin = ogmrip_plugin_find_container_by_type (container_plugins, container);
+  if (!plugin)
+    return -1;
+
+  return plugin->max_subp;
+}
+
+/**
+ * ogmrip_plugin_get_container_module:
+ * @container: A container type
+ *
+ * Gets the #GModule associated with @container.
+ *
+ * Returns: A #GModule, or NULL
+ */
+GModule *
+ogmrip_plugin_get_container_module (GType container)
+{
+  OGMRipContainerPlugin *plugin;
+
+  g_return_val_if_fail (g_type_is_a (container, OGMRIP_TYPE_CONTAINER), NULL);
+
+  plugin = ogmrip_plugin_find_container_by_type (container_plugins, container);
+  if (!plugin)
+    return NULL;
+
+  return plugin->module;
+}
+
 /**
  * ogmrip_plugin_get_n_video_codecs:
  *
@@ -663,6 +808,28 @@ ogmrip_plugin_get_video_codec_threads (GType codec)
   return ((OGMRipVideoPlugin *) plugin)->threads;
 }
 
+/**
+ * ogmrip_plugin_get_video_codec_module:
+ * @codec: A video codec type
+ *
+ * Gets the #GModule associated with @codec.
+ *
+ * Returns: A #GModule, or NULL
+ */
+GModule *
+ogmrip_plugin_get_video_codec_module (GType codec)
+{
+  OGMRipPlugin *plugin;
+
+  g_return_val_if_fail (g_type_is_a (codec, OGMRIP_TYPE_VIDEO_CODEC), NULL);
+
+  plugin = ogmrip_plugin_find_codec_by_type (video_plugins, codec);
+  if (!plugin)
+    return NULL;
+
+  return plugin->module;
+}
+
 /**
  * ogmrip_plugin_get_n_audio_codecs:
  *
@@ -783,6 +950,28 @@ ogmrip_plugin_get_audio_codec_format (GType codec)
   return ogmrip_plugin_get_codec_format (audio_plugins, codec);
 }
 
+/**
+ * ogmrip_plugin_get_audio_codec_module:
+ * @codec: A audio codec type
+ *
+ * Gets the #GModule associated with @codec.
+ *
+ * Returns: A #GModule, or NULL
+ */
+GModule *
+ogmrip_plugin_get_audio_codec_module (GType codec)
+{
+  OGMRipPlugin *plugin;
+
+  g_return_val_if_fail (g_type_is_a (codec, OGMRIP_TYPE_AUDIO_CODEC), NULL);
+
+  plugin = ogmrip_plugin_find_codec_by_type (audio_plugins, codec);
+  if (!plugin)
+    return NULL;
+
+  return plugin->module;
+}
+
 /**
  * ogmrip_plugin_get_n_subp_codecs:
  *
@@ -868,7 +1057,7 @@ ogmrip_plugin_get_subp_codec_by_name (const gchar *name)
 gint
 ogmrip_plugin_get_subp_codec_index (GType codec)
 {
-  g_return_val_if_fail (g_type_is_a (codec, OGMRIP_TYPE_SUBP_CODEC), 0);
+  g_return_val_if_fail (g_type_is_a (codec, OGMRIP_TYPE_SUBP_CODEC), -1);
 
   return ogmrip_plugin_get_codec_index (subp_plugins, codec);
 }
@@ -927,6 +1116,28 @@ ogmrip_plugin_get_subp_codec_text (GType codec)
   return ((OGMRipSubpPlugin *) plugin)->text;
 }
 
+/**
+ * ogmrip_plugin_get_subp_codec_module:
+ * @codec: A subp codec type
+ *
+ * Gets the #GModule associated with @codec.
+ *
+ * Returns: A #GModule, or NULL
+ */
+GModule *
+ogmrip_plugin_get_subp_codec_module (GType codec)
+{
+  OGMRipPlugin *plugin;
+
+  g_return_val_if_fail (g_type_is_a (codec, OGMRIP_TYPE_SUBP_CODEC), NULL);
+
+  plugin = ogmrip_plugin_find_codec_by_type (subp_plugins, codec);
+  if (!plugin)
+    return NULL;
+
+  return plugin->module;
+}
+
 /**
  * ogmrip_plugin_can_contain_format:
  * @container: A container type
@@ -975,7 +1186,18 @@ gboolean
 ogmrip_plugin_can_contain_video (GType container, GType codec)
 {
   g_return_val_if_fail (g_type_is_a (container, OGMRIP_TYPE_CONTAINER), FALSE);
-  g_return_val_if_fail (g_type_is_a (codec, OGMRIP_TYPE_VIDEO_CODEC), FALSE);
+  g_return_val_if_fail (codec == G_TYPE_NONE || g_type_is_a (codec, OGMRIP_TYPE_VIDEO_CODEC), FALSE);
+
+  if (codec == G_TYPE_NONE)
+  {
+    OGMRipContainerPlugin *plugin;
+
+    plugin = ogmrip_plugin_find_container_by_type (container_plugins, container);
+    if (!plugin)
+      return FALSE;
+
+    return plugin->video ? FALSE : TRUE;
+  }
 
   return ogmrip_plugin_can_contain_codec (container, video_plugins, codec);
 }
@@ -1013,6 +1235,9 @@ ogmrip_plugin_can_contain_subp (GType container, GType codec)
   g_return_val_if_fail (g_type_is_a (container, OGMRIP_TYPE_CONTAINER), FALSE);
   g_return_val_if_fail (g_type_is_a (codec, OGMRIP_TYPE_SUBP_CODEC), FALSE);
 
+  if (codec == OGMRIP_TYPE_HARDSUB)
+    return TRUE;
+
   return ogmrip_plugin_can_contain_codec (container, subp_plugins, codec);
 }
 
@@ -1062,47 +1287,3 @@ ogmrip_plugin_can_contain_n_subp (GType container, guint ncodec)
   return ncodec <= plugin->max_subp;
 }
 
-/**
- * ogmrip_plugin_get_container_max_audio:
- * @container: A container type
- *
- * Returns the number of audio streams the given container can contain.
- *
- * Returns: the number of audio streams, or -1
- */
-gint
-ogmrip_plugin_get_container_max_audio (GType container)
-{
-  OGMRipContainerPlugin *plugin;
-
-  g_return_val_if_fail (g_type_is_a (container, OGMRIP_TYPE_CONTAINER), -1);
-
-  plugin = ogmrip_plugin_find_container_by_type (container_plugins, container);
-  if (!plugin)
-    return FALSE;
-
-  return plugin->max_audio;
-}
-
-/**
- * ogmrip_plugin_get_container_max_subp:
- * @container: A container type
- *
- * Returns the number of subtitle streams the given container can contain.
- *
- * Returns: the number of subtitle streams, or -1
- */
-gint
-ogmrip_plugin_get_container_max_subp (GType container)
-{
-  OGMRipContainerPlugin *plugin;
-
-  g_return_val_if_fail (g_type_is_a (container, OGMRIP_TYPE_CONTAINER), -1);
-
-  plugin = ogmrip_plugin_find_container_by_type (container_plugins, container);
-  if (!plugin)
-    return FALSE;
-
-  return plugin->max_subp;
-}
-
