@@ -199,16 +199,19 @@ ogmrip_check_mplayer_dts (void)
 
   if (!dts_checked)
   {
-    gchar *output;
+    gchar *output, *error;
 
     dts_checked = TRUE;
 
     if (!ogmrip_check_mplayer ())
       return FALSE;
 
-    if (!g_spawn_command_line_sync ("mplayer -ac help", &output, NULL, NULL, NULL))
+    if (!g_spawn_command_line_sync (MPLAYER_CHECK_VERSION (1,0,3,0) ?
+          "mplayer -ac help -noconfig all" : "mplayer -ac help", &output, &error, NULL, NULL))
       return FALSE;
 
+    g_free (error);
+
     have_dts = g_regex_match_simple ("^(ffdts|ffdca|dts).*working.*$", output, G_REGEX_MULTILINE, 0);
     g_free (output);
   }
