@@ -38,7 +38,8 @@
 #define ROUND(x) ((gint) ((x) + 0.5) != (gint) (x) ? ((gint) ((x) + 0.5)) : ((gint) (x)))
 #define FLOOR(x) ((gint) (x))
 
-#define MAX_FRAMES 100
+#define CROP_FRAMES    100
+#define ANALYZE_FRAMES 500
 
 struct _OGMRipVideoCodecPriv
 {
@@ -46,7 +47,6 @@ struct _OGMRipVideoCodecPriv
   gdouble quantizer;
   gint bitrate;
   guint angle;
-  guint nframes;
   guint passes;
   guint threads;
   guint crop_x;
@@ -57,6 +57,8 @@ struct _OGMRipVideoCodecPriv
   guint scale_height;
   guint max_width;
   guint max_height;
+  guint aspect_num;
+  guint aspect_denom;
   guint max_b_frames;
   gboolean grayscale;
   gboolean cartoon;
@@ -68,12 +70,16 @@ struct _OGMRipVideoCodecPriv
   gboolean qpel;
   gboolean v4mv;
   gboolean expand;
+
   OGMDvdAudioStream *astream;
+  OGMDvdSubpStream *sstream;
   OGMRipQualityType quality;
   OGMRipScalerType scaler;
   OGMRipDeintType deint;
-  OGMJobSpawn *crop_child;
-  gboolean crop_canceled;
+
+  OGMJobSpawn *child;
+  gboolean child_canceled;
+  gboolean forced_subs;
 };
 
 typedef struct
@@ -84,7 +90,7 @@ typedef struct
 
 typedef struct
 {
-  OGMRipVideoCodec *video;
+  guint nframes;
   guint frames;
   GSList *x;
   GSList *y;
@@ -92,6 +98,17 @@ typedef struct
   GSList *h;
 } OGMRipCrop;
 
+typedef struct
+{
+  gboolean content;
+  gboolean progressive;
+  gboolean telecine;
+  gint affinity;
+  gint duration;
+  guint nframes;
+  guint frames;
+} OGMRipAnalyze;
+
 enum 
 {
   PROP_0,
@@ -124,7 +141,6 @@ static void ogmrip_video_codec_get_property (GObject      *gobject,
                                              GParamSpec   *pspec);
 static void ogmrip_video_codec_cancel       (OGMJobSpawn  *spawn);
 
-// static const gchar *deinterlacer[] = { "lb", "li", "ci", "md", "fd", "l5", "kerndeint", "yadif=3:1,mcdeint=2:1:10" };
 static const gchar *deinterlacer[] = { "lb", "li", "ci", "md", "fd", "l5", "kerndeint", "yadif=0" };
 
 static gint
@@ -133,8 +149,20 @@ g_ulist_compare (UInfo *info, gint val)
   return info->val - val;
 }
 
+static gint
+g_ulist_min (UInfo *info1, UInfo *info2)
+{
+  return info2->val - info1->val;
+}
+
+static gint
+g_ulist_max (UInfo *info1, UInfo *info2)
+{
+  return info1->val - info2->val;
+}
+
 static GSList *
-g_ulist_add (GSList *ulist, gint val)
+g_ulist_add (GSList *ulist, GCompareFunc func, gint val)
 {
   GSList *ulink;
   UInfo *info;
@@ -151,7 +179,7 @@ g_ulist_add (GSList *ulist, gint val)
     info->val = val;
     info->ref = 1;
 
-    ulist = g_slist_append (ulist, info);
+    ulist = g_slist_insert_sorted (ulist, info, func);
   }
 
   return ulist;
@@ -201,23 +229,23 @@ ogmrip_video_codec_crop_watch (OGMJobExec *exec, const gchar *buffer, OGMRipCrop
     if (sscanf (str, "-vf crop=%d:%d:%d:%d", &w, &h, &x, &y) == 4)
     {
       if (w > 0)
-        info->w = g_ulist_add (info->w, w);
+        info->w = g_ulist_add (info->w, (GCompareFunc) g_ulist_min, w);
       if (h > 0)
-        info->h = g_ulist_add (info->h, h);
+        info->h = g_ulist_add (info->h, (GCompareFunc) g_ulist_min, h);
       if (x > 0)
-        info->x = g_ulist_add (info->x, x);
+        info->x = g_ulist_add (info->x, (GCompareFunc) g_ulist_max, x);
       if (y > 0)
-        info->y = g_ulist_add (info->y, y);
+        info->y = g_ulist_add (info->y, (GCompareFunc) g_ulist_max, y);
     }
 
     frame ++;
-    if (frame == info->video->priv->nframes - 2)
+    if (frame == info->nframes - 2)
     {
       frame = 0;
       return 1.0;
     }
 
-    return frame / (gdouble) (info->video->priv->nframes - 2);
+    return frame / (gdouble) (info->nframes - 2);
   }
   else
   {
@@ -227,7 +255,7 @@ ogmrip_video_codec_crop_watch (OGMJobExec *exec, const gchar *buffer, OGMRipCrop
     {
       info->frames ++;
 
-      if (info->frames >= MAX_FRAMES)
+      if (info->frames >= CROP_FRAMES)
         ogmjob_spawn_cancel (OGMJOB_SPAWN (exec));
     }
   }
@@ -235,6 +263,58 @@ ogmrip_video_codec_crop_watch (OGMJobExec *exec, const gchar *buffer, OGMRipCrop
   return -1.0;
 }
 
+static gdouble
+ogmrip_video_codec_analyze_watch (OGMJobExec *exec, const gchar *buffer, OGMRipAnalyze *info)
+{
+  if (g_str_has_prefix (buffer, "V: "))
+  {
+    info->frames ++;
+
+    if (info->frames == info->nframes)
+      return 1.0;
+
+    return info->frames / (gdouble) info->nframes;
+  }
+  else
+  {
+    if (!info->progressive)
+      info->progressive = g_str_has_prefix (buffer, "demux_mpg:") && strstr (buffer, "24000/1001");
+
+    if (!info->content)
+    {
+      gint duration;
+
+      if (sscanf (buffer, "duration: %d", &duration) == 1)
+      {
+        info->content = info->duration == 3 && duration == 2;
+        info->duration = duration;
+      }
+    }
+    else
+    {
+      if (!info->telecine)
+      {
+        if (g_str_has_prefix (buffer, "affinity: "))
+        {
+          gint affinity;
+
+          if (g_str_has_prefix (buffer + 10, ".0+.1.+2"))
+            affinity = 1;
+          else if (g_str_has_prefix (buffer + 10, ".0++1"))
+            affinity = 2;
+          else
+            affinity = 0;
+
+          info->telecine = info->affinity == 1 && affinity == 2;
+          info->affinity = affinity;
+        }
+      }
+    }
+  }
+
+  return -1.0;
+}
+
 static gchar **
 ogmrip_video_codec_crop_command (OGMRipVideoCodec *video, gdouble start, gulong nframes)
 {
@@ -272,6 +352,12 @@ ogmrip_video_codec_crop_command (OGMRipVideoCodec *video, gdouble start, gulong
   g_ptr_array_add (argv, g_strdup ("-nosound"));
   g_ptr_array_add (argv, g_strdup ("-nocache"));
 
+  if (MPLAYER_CHECK_VERSION (1,0,3,0))
+  {
+    g_ptr_array_add (argv, g_strdup ("-noconfig"));
+    g_ptr_array_add (argv, g_strdup ("all"));
+  }
+
   g_ptr_array_add (argv, g_strdup ("-speed"));
   g_ptr_array_add (argv, g_strdup ("100"));
 
@@ -322,6 +408,68 @@ ogmrip_video_codec_crop_command (OGMRipVideoCodec *video, gdouble start, gulong
   return (gchar **) g_ptr_array_free (argv, FALSE);
 }
 
+static gchar **
+ogmrip_video_codec_analyze_command (OGMRipVideoCodec *video, gulong nframes)
+{
+  OGMDvdTitle *title;
+  GPtrArray *argv;
+
+  const gchar *device;
+  gint vid;
+
+  g_return_val_if_fail (OGMRIP_IS_VIDEO_CODEC (video), NULL);
+
+  title = ogmrip_codec_get_input (OGMRIP_CODEC (video));
+  g_return_val_if_fail (title != NULL, NULL);
+
+  argv = g_ptr_array_new ();
+
+  g_ptr_array_add (argv, g_strdup ("mplayer"));
+  g_ptr_array_add (argv, g_strdup ("-nolirc"));
+  g_ptr_array_add (argv, g_strdup ("-nosound"));
+  g_ptr_array_add (argv, g_strdup ("-nocache"));
+
+  if (MPLAYER_CHECK_VERSION (1,0,3,0))
+  {
+    g_ptr_array_add (argv, g_strdup ("-noconfig"));
+    g_ptr_array_add (argv, g_strdup ("all"));
+  }
+
+  g_ptr_array_add (argv, g_strdup ("-v"));
+  g_ptr_array_add (argv, g_strdup ("-benchmark"));
+
+  g_ptr_array_add (argv, g_strdup ("-vo"));
+  g_ptr_array_add (argv, g_strdup ("null"));
+
+  g_ptr_array_add (argv, g_strdup ("-vf"));
+  g_ptr_array_add (argv, g_strdup ("pullup"));
+
+  g_ptr_array_add (argv, g_strdup ("-dvdangle"));
+  g_ptr_array_add (argv, g_strdup_printf ("%d",
+        ogmrip_video_codec_get_angle (video)));
+
+  g_ptr_array_add (argv, g_strdup ("-frames"));
+  g_ptr_array_add (argv, g_strdup_printf ("%lu", nframes));
+
+  device = ogmdvd_disc_get_device (ogmdvd_title_get_disc (title));
+  g_ptr_array_add (argv, g_strdup ("-dvd-device"));
+  g_ptr_array_add (argv, g_strdup (device));
+
+  vid = ogmdvd_title_get_nr (title);
+
+  if (MPLAYER_CHECK_VERSION (1,0,0,1))
+    g_ptr_array_add (argv, g_strdup_printf ("dvd://%d", vid + 1));
+  else
+  {
+    g_ptr_array_add (argv, g_strdup ("-dvd"));
+    g_ptr_array_add (argv, g_strdup_printf ("%d", vid + 1));
+  }
+
+  g_ptr_array_add (argv, NULL);
+
+  return (gchar **) g_ptr_array_free (argv, FALSE);
+}
+
 G_DEFINE_ABSTRACT_TYPE (OGMRipVideoCodec, ogmrip_video_codec, OGMRIP_TYPE_CODEC)
 
 static void
@@ -602,10 +750,10 @@ ogmrip_video_codec_cancel (OGMJobSpawn *spawn)
 
   video = OGMRIP_VIDEO_CODEC (spawn);
 
-  if (video->priv->crop_child)
+  if (video->priv->child)
   {
-    ogmjob_spawn_cancel (video->priv->crop_child);
-    video->priv->crop_canceled = TRUE;
+    ogmjob_spawn_cancel (video->priv->child);
+    video->priv->child_canceled = TRUE;
   }
 
   OGMJOB_SPAWN_CLASS (ogmrip_video_codec_parent_class)->cancel (spawn);
@@ -736,7 +884,7 @@ ogmrip_video_codec_set_quantizer (OGMRipVideoCodec *video, gdouble quantizer)
 {
   g_return_if_fail (OGMRIP_IS_VIDEO_CODEC (video));
 
-  video->priv->quantizer = CLAMP (quantizer, 1, 31);
+  video->priv->quantizer = CLAMP (quantizer, 0, 31);
   video->priv->bitrate = -1;
 }
 
@@ -1174,7 +1322,7 @@ ogmrip_video_codec_set_quality (OGMRipVideoCodec *video, OGMRipQualityType quali
 
   g_return_if_fail (OGMRIP_IS_VIDEO_CODEC (video));
 
-  video->priv->quality = CLAMP (quality, OGMRIP_QUALITY_EXTREME, OGMRIP_QUALITY_NORMAL);
+  video->priv->quality = CLAMP (quality, OGMRIP_QUALITY_EXTREME, OGMRIP_QUALITY_USER);
 
   klass = OGMRIP_VIDEO_CODEC_GET_CLASS (video);
 
@@ -1425,10 +1573,7 @@ ogmrip_video_codec_get_scale_size (OGMRipVideoCodec *video, guint *width, guint
   if (height)
     *height = scale_height;
 
-  if (scale_width == raw_width && scale_height == raw_height)
-    return FALSE;
-
-  return TRUE;
+  return scale_width != raw_width || scale_height != raw_height;
 }
 
 /**
@@ -1476,10 +1621,7 @@ ogmrip_video_codec_get_max_size (OGMRipVideoCodec *video, guint *width, guint *h
   if (expand)
     *expand = video->priv->expand;
 
-  if (!video->priv->max_width || !video->priv->max_height)
-    return FALSE;
-
-  return TRUE;
+  return video->priv->max_width && video->priv->max_height;
 }
 
 /**
@@ -1504,6 +1646,51 @@ ogmrip_video_codec_set_max_size (OGMRipVideoCodec *video, guint width, guint hei
   ogmrip_video_codec_autosize (video);
 }
 
+/**
+ * ogmrip_video_codec_get_aspect_ratio:
+ * @video: an #OGMRipVideoCodec
+ * @num: a pointer to store the numerator of the aspect ratio
+ * @denom: a pointer to store the denominator of the aspect ratio
+ *
+ * Gets the aspect ratio of the movie.
+ */
+void
+ogmrip_video_codec_get_aspect_ratio (OGMRipVideoCodec *video, guint *num, guint *denom)
+{
+  g_return_if_fail (OGMRIP_IS_VIDEO_CODEC (video));
+
+  if (!video->priv->aspect_num || !video->priv->aspect_denom)
+  {
+    OGMDvdTitle *title;
+
+    title = ogmrip_codec_get_input (OGMRIP_CODEC (video));
+    ogmdvd_title_get_aspect_ratio (title, &video->priv->aspect_num, &video->priv->aspect_denom);
+  }
+
+  if (num)
+    *num = video->priv->aspect_num;
+
+  if (denom)
+    *denom = video->priv->aspect_denom;
+}
+
+/**
+ * ogmrip_video_codec_set_aspect_ratio:
+ * @video: an #OGMRipVideoCodec
+ * @num: the numerator of the aspect ratio
+ * @denom: the denominator of the aspect ratio
+ *
+ * Sets the aspect ratio of the movie.
+ */
+void
+ogmrip_video_codec_set_aspect_ratio (OGMRipVideoCodec *video, guint num, guint denom)
+{
+  g_return_if_fail (OGMRIP_IS_VIDEO_CODEC (video));
+
+  video->priv->aspect_num = num;
+  video->priv->aspect_denom = denom;
+}
+
 /**
  * ogmrip_video_codec_autoscale:
  * @video: an #OGMRipVideoCodec
@@ -1527,11 +1714,11 @@ ogmrip_video_codec_autoscale (OGMRipVideoCodec *video)
   g_return_if_fail (title != NULL);
 
   ogmrip_video_codec_get_raw_size (video, &raw_width, &raw_height);
+  ogmrip_video_codec_get_aspect_ratio (video, &anumerator, &adenominator);
 
   crop_width = video->priv->crop_width > 0 ? video->priv->crop_width : raw_width;
   crop_height = video->priv->crop_height > 0 ? video->priv->crop_height : raw_height;
 
-  ogmdvd_title_get_aspect_ratio (title, &anumerator, &adenominator);
   ogmdvd_title_get_framerate (title, &rnumerator, &rdenominator);
 
   ratio = (crop_width * raw_height * anumerator) / (gdouble) (crop_height * raw_width * adenominator);
@@ -1588,7 +1775,7 @@ ogmrip_video_codec_autobitrate (OGMRipVideoCodec *video, guint64 nonvideo_size,
 }
 
 static void
-ogmrip_video_codec_autocrop_progress (OGMJobSpawn *spawn, gdouble fraction, OGMRipVideoCodec *video)
+ogmrip_video_codec_child_progress (OGMJobSpawn *spawn, gdouble fraction, OGMRipVideoCodec *video)
 {
   g_signal_emit_by_name (video, "progress", fraction);
 }
@@ -1614,46 +1801,45 @@ ogmrip_video_codec_autocrop (OGMRipVideoCodec *video, guint nframes)
 
   g_return_val_if_fail (OGMRIP_IS_VIDEO_CODEC (video), FALSE);
 
+  memset (&crop, 0, sizeof (OGMRipCrop));
+
   if (!nframes)
   {
     if (MPLAYER_CHECK_VERSION (1,0,0,8))
-      video->priv->nframes = 12;
+      crop.nframes = 12;
     else
-      video->priv->nframes = 30;
+      crop.nframes = 30;
   }
   else
-    video->priv->nframes = nframes + 5;
+    crop.nframes = nframes + 5;
 
-  video->priv->crop_child = ogmjob_queue_new ();
+  video->priv->child = ogmjob_queue_new ();
 
-  g_signal_connect (video->priv->crop_child, "progress",
-      G_CALLBACK (ogmrip_video_codec_autocrop_progress), video);
+  g_signal_connect (video->priv->child, "progress",
+      G_CALLBACK (ogmrip_video_codec_child_progress), video);
 
   length = ogmrip_codec_get_length (OGMRIP_CODEC (video), NULL);
   step = length / 5.;
 
-  memset (&crop, 0, sizeof (OGMRipCrop));
-  crop.video = video;
-
   for (start = step; start < length; start += step)
   {
-    argv = ogmrip_video_codec_crop_command (video, start, video->priv->nframes);
+    argv = ogmrip_video_codec_crop_command (video, start, crop.nframes);
 
     child = ogmjob_exec_newv (argv);
-    ogmjob_container_add (OGMJOB_CONTAINER (video->priv->crop_child), child);
+    ogmjob_container_add (OGMJOB_CONTAINER (video->priv->child), child);
     g_object_unref (child);
 
     ogmjob_exec_add_watch_full (OGMJOB_EXEC (child),
         (OGMJobWatch) ogmrip_video_codec_crop_watch, &crop, TRUE, FALSE, FALSE);
   }
 
-  video->priv->crop_canceled = FALSE;
+  video->priv->child_canceled = FALSE;
 
-  ogmjob_spawn_run (video->priv->crop_child, NULL);
-  g_object_unref (video->priv->crop_child);
-  video->priv->crop_child = NULL;
+  ogmjob_spawn_run (video->priv->child, NULL);
+  g_object_unref (video->priv->child);
+  video->priv->child = NULL;
 
-  if (video->priv->crop_canceled)
+  if (video->priv->child_canceled)
     return FALSE;
 
   w = g_ulist_get_most_frequent (crop.w);
@@ -1673,3 +1859,100 @@ ogmrip_video_codec_autocrop (OGMRipVideoCodec *video, guint nframes)
   return TRUE;
 }
 
+/**
+ * ogmrip_video_codec_analyze:
+ * @video: an #OGMRipVideoCodec
+ * @nframes: the number of frames
+ *
+ * Analyze the video stream to detect if the video is progressive,
+ * interlaced and/or telecine.
+ *
+ * Returns: %FALSE, on error or cancel
+ */
+gboolean
+ogmrip_video_codec_analyze (OGMRipVideoCodec *video, guint nframes)
+{
+  OGMRipAnalyze analyze;
+  gchar **argv;
+
+  g_return_val_if_fail (OGMRIP_IS_VIDEO_CODEC (video), FALSE);
+
+  if (!nframes)
+    nframes = ANALYZE_FRAMES;
+
+  memset (&analyze, 0, sizeof (OGMRipAnalyze));
+  analyze.nframes = nframes;
+
+  argv = ogmrip_video_codec_analyze_command (video, analyze.nframes);
+
+  video->priv->child = ogmjob_exec_newv (argv);
+
+  g_signal_connect (video->priv->child, "progress",
+      G_CALLBACK (ogmrip_video_codec_child_progress), video);
+
+  ogmjob_exec_add_watch_full (OGMJOB_EXEC (video->priv->child),
+      (OGMJobWatch) ogmrip_video_codec_analyze_watch, &analyze, TRUE, FALSE, FALSE);
+
+  video->priv->child_canceled = FALSE;
+
+  ogmjob_spawn_run (video->priv->child, NULL);
+  g_object_unref (video->priv->child);
+  video->priv->child = NULL;
+
+  if (video->priv->child_canceled)
+    return FALSE;
+
+  ogmrip_video_codec_set_deinterlacer (video,
+      analyze.content && !analyze.telecine ? OGMRIP_DEINT_YADIF: OGMRIP_DEINT_NONE);
+  ogmrip_codec_set_telecine (OGMRIP_CODEC (video), analyze.content && analyze.telecine);
+  ogmrip_codec_set_progressive (OGMRIP_CODEC (video), analyze.progressive);
+
+  return TRUE;
+}
+
+/**
+ * ogmrip_video_codec_get_hard_subp:
+ * @video: an #OGMRipVideoCodec
+ * @forced: location to store whether to hardcode forced subs only
+ *
+ * Gets the subp stream that will be hardcoded in the video.
+ *
+ * Returns: the #OGMDvdSubpStream, or NULL
+ */
+OGMDvdSubpStream *
+ogmrip_video_codec_get_hard_subp (OGMRipVideoCodec *video, gboolean *forced)
+{
+  g_return_val_if_fail (OGMRIP_IS_VIDEO_CODEC (video), NULL);
+
+  if (forced)
+    *forced = video->priv->forced_subs;
+
+  return video->priv->sstream;
+}
+
+/**
+ * ogmrip_video_codec_set_hard_subp:
+ * @video: an #OGMRipVideoCodec
+ * @stream: an #OGMDvdSubpStream
+ * @forced: whether to hardcode forced subs only
+ *
+ * Sets the subp stream that will be hardcoded in the video.
+ */
+void
+ogmrip_video_codec_set_hard_subp (OGMRipVideoCodec *video, OGMDvdSubpStream *stream, gboolean forced)
+{
+  g_return_if_fail (OGMRIP_IS_VIDEO_CODEC (video));
+
+  if (video->priv->sstream != stream)
+  {
+    if (stream)
+      ogmdvd_stream_ref (OGMDVD_STREAM (stream));
+
+    if (video->priv->sstream)
+      ogmdvd_stream_unref (OGMDVD_STREAM (video->priv->sstream));
+
+    video->priv->sstream = stream;
+    video->priv->forced_subs = forced;
+  }
+}
+
