@@ -143,6 +143,14 @@ void      ogmrip_video_codec_set_max_size         (OGMRipVideoCodec  *video,
                                                    guint             width,
                                                    guint             height,
                                                    gboolean          expand);
+void      ogmrip_video_codec_get_aspect_ratio     (OGMRipVideoCodec  *video,
+                                                   guint             *num,
+                                                   guint             *denom);
+void      ogmrip_video_codec_set_aspect_ratio     (OGMRipVideoCodec  *video,
+                                                   guint             num,
+                                                   guint             denom);
+gboolean  ogmrip_video_codec_analyze              (OGMRipVideoCodec  *video,
+                                                   guint             nframes);
 gboolean  ogmrip_video_codec_autocrop             (OGMRipVideoCodec  *video,
                                                    guint             nframes);
 void      ogmrip_video_codec_autoscale            (OGMRipVideoCodec  *video);
@@ -155,6 +163,12 @@ OGMDvdAudioStream * ogmrip_video_codec_get_ensure_sync (OGMRipVideoCodec  *video
 void                ogmrip_video_codec_set_ensure_sync (OGMRipVideoCodec  *video,
                                                         OGMDvdAudioStream *stream);
 
+OGMDvdSubpStream *  ogmrip_video_codec_get_hard_subp   (OGMRipVideoCodec  *video,
+                                                        gboolean          *forced);
+void                ogmrip_video_codec_set_hard_subp   (OGMRipVideoCodec  *video,
+                                                        OGMDvdSubpStream  *stream,
+                                                        gboolean          forced);
+
 G_END_DECLS
 
 #endif /* __OGMRIP_VIDEO_CODEC_H__ */
