@@ -31,21 +31,22 @@ G_BEGIN_DECLS
 
 #define OGMRIP_GCONF_GENERAL                 OGMRIP_GCONF_ROOT "/general"
 
-#define OGMRIP_GCONF_PROFILE                 OGMRIP_GCONF_GENERAL "/profile"
-#define OGMRIP_GCONF_OUTPUT_DIR              OGMRIP_GCONF_GENERAL "/output_dir"
-#define OGMRIP_GCONF_FILENAME                OGMRIP_GCONF_GENERAL "/filename"
-#define OGMRIP_GCONF_PREF_AUDIO              OGMRIP_GCONF_GENERAL "/pref_audio"
-#define OGMRIP_GCONF_PREF_SUBP               OGMRIP_GCONF_GENERAL "/pref_subp"
-#define OGMRIP_GCONF_CHAPTER_LANG            OGMRIP_GCONF_GENERAL "/chapter_lang"
-#define OGMRIP_GCONF_EXPERT_MODE             OGMRIP_GCONF_GENERAL "/expert"
+#define OGMRIP_GCONF_PROFILE                 "profile"
+#define OGMRIP_GCONF_OUTPUT_DIR              "output_dir"
+#define OGMRIP_GCONF_FILENAME                "filename"
+#define OGMRIP_GCONF_PREF_AUDIO              "pref_audio"
+#define OGMRIP_GCONF_PREF_SUBP               "pref_subp"
+#define OGMRIP_GCONF_CHAPTER_LANG            "chapter_lang"
 
 #define OGMRIP_GCONF_ADVANCED                OGMRIP_GCONF_ROOT "/advanced"
 
-#define OGMRIP_GCONF_TMP_DIR                 OGMRIP_GCONF_GENERAL "/tmp_dir"
-#define OGMRIP_GCONF_COPY_DVD                OGMRIP_GCONF_ADVANCED "/copy_dvd"
-#define OGMRIP_GCONF_AFTER_ENC               OGMRIP_GCONF_ADVANCED "/after_enc"
-#define OGMRIP_GCONF_KEEP_TMP                OGMRIP_GCONF_ADVANCED "/keep_tmp"
-#define OGMRIP_GCONF_LOG_OUTPUT              OGMRIP_GCONF_ADVANCED "/log_output"
+#define OGMRIP_GCONF_TMP_DIR                 "tmp_dir"
+#define OGMRIP_GCONF_COPY_DVD                "copy_dvd"
+#define OGMRIP_GCONF_AFTER_ENC               "after_enc"
+#define OGMRIP_GCONF_KEEP_TMP                "keep_tmp"
+#define OGMRIP_GCONF_LOG_OUTPUT              "log_output"
+#define OGMRIP_GCONF_THREADS                 "threads"
+#define OGMRIP_GCONF_AUTO_SUBP               "auto_subp"
 
 #define OGMRIP_GCONF_PROFILES                OGMRIP_GCONF_ROOT "/profiles"
 
@@ -69,17 +70,18 @@ G_BEGIN_DECLS
 #define OGMRIP_GCONF_VIDEO_PASSES            OGMRIP_GCONF_VIDEO "/passes"
 #define OGMRIP_GCONF_VIDEO_PRESET            OGMRIP_GCONF_VIDEO "/preset"
 #define OGMRIP_GCONF_VIDEO_SCALER            OGMRIP_GCONF_VIDEO "/scaler"
-#define OGMRIP_GCONF_VIDEO_THREADS           OGMRIP_GCONF_VIDEO "/threads"
 #define OGMRIP_GCONF_VIDEO_DENOISE           OGMRIP_GCONF_VIDEO "/denoise"
 #define OGMRIP_GCONF_VIDEO_TRELLIS           OGMRIP_GCONF_VIDEO "/trellis"
 #define OGMRIP_GCONF_VIDEO_QPEL              OGMRIP_GCONF_VIDEO "/qpel"
 #define OGMRIP_GCONF_VIDEO_DEBLOCK           OGMRIP_GCONF_VIDEO "/deblock"
 #define OGMRIP_GCONF_VIDEO_DERING            OGMRIP_GCONF_VIDEO "/dering"
 #define OGMRIP_GCONF_VIDEO_TURBO             OGMRIP_GCONF_VIDEO "/turbo"
+#define OGMRIP_GCONF_VIDEO_ASPECT            OGMRIP_GCONF_VIDEO "/aspect_ratio"
 
 #define OGMRIP_GCONF_VIDEO_ENCODING          OGMRIP_GCONF_VIDEO "/encoding"
 #define OGMRIP_GCONF_VIDEO_BITRATE           OGMRIP_GCONF_VIDEO "/bitrate"
 #define OGMRIP_GCONF_VIDEO_QUANTIZER         OGMRIP_GCONF_VIDEO "/quantizer"
+#define OGMRIP_GCONF_VIDEO_BPP               OGMRIP_GCONF_VIDEO "/bpp"
 
 #define OGMRIP_GCONF_VIDEO_CAN_CROP          OGMRIP_GCONF_VIDEO "/can_crop"
 #define OGMRIP_GCONF_VIDEO_CAN_SCALE         OGMRIP_GCONF_VIDEO "/can_scale"
@@ -99,7 +101,7 @@ G_BEGIN_DECLS
 
 #define OGMRIP_GCONF_SUBP_CODEC              OGMRIP_GCONF_SUBP "/codec"
 #define OGMRIP_GCONF_SUBP_CHARSET            OGMRIP_GCONF_SUBP "/charset"
-#define OGMRIP_GCONF_SUBP_EOL                OGMRIP_GCONF_SUBP "/eol"
+#define OGMRIP_GCONF_SUBP_NEWLINE            OGMRIP_GCONF_SUBP "/newline"
 #define OGMRIP_GCONF_FORCED_SUBS             OGMRIP_GCONF_SUBP "/forced"
 #define OGMRIP_GCONF_SPELL_CHECK             OGMRIP_GCONF_SUBP "/spell_check"
 
@@ -113,13 +115,14 @@ G_BEGIN_DECLS
 #define OGMRIP_DEFAULT_PREF_AUDIO            0
 #define OGMRIP_DEFAULT_PREF_SUBP             0
 #define OGMRIP_DEFAULT_CHAPTER_LANG          0
-#define OGMRIP_DEFAULT_EXPERT_MODE           FALSE
 
 #define OGMRIP_DEFAULT_TMP_DIR               ogmrip_fs_get_tmp_dir ()
 #define OGMRIP_DEFAULT_COPY_DVD              FALSE
 #define OGMRIP_DEFAULT_AFTER_ENC             0
 #define OGMRIP_DEFAULT_KEEP_TMP              FALSE
 #define OGMRIP_DEFAULT_LOG_OUTPUT            FALSE
+#define OGMRIP_DEFAULT_THREADS               1
+#define OGMRIP_DEFAULT_AUTO_SUBP             TRUE
 
 #define OGMRIP_DEFAULT_CONTAINER_FORMAT      "ogm"
 #define OGMRIP_DEFAULT_CONTAINER_FOURCC      0
@@ -131,23 +134,24 @@ G_BEGIN_DECLS
 #define OGMRIP_DEFAULT_VIDEO_PASSES          2
 #define OGMRIP_DEFAULT_VIDEO_PRESET          0
 #define OGMRIP_DEFAULT_VIDEO_SCALER          7
-#define OGMRIP_DEFAULT_VIDEO_THREADS         1
 #define OGMRIP_DEFAULT_VIDEO_DENOISE         TRUE
 #define OGMRIP_DEFAULT_VIDEO_TRELLIS         TRUE
 #define OGMRIP_DEFAULT_VIDEO_QPEL            FALSE
 #define OGMRIP_DEFAULT_VIDEO_DEBLOCK         FALSE
 #define OGMRIP_DEFAULT_VIDEO_DERING          FALSE
 #define OGMRIP_DEFAULT_VIDEO_TURBO           TRUE
+#define OGMRIP_DEFAULT_VIDEO_ASPECT          2
 
 #define OGMRIP_DEFAULT_VIDEO_ENCODING        0
 #define OGMRIP_DEFAULT_VIDEO_BITRATE         800
 #define OGMRIP_DEFAULT_VIDEO_QUANTIZER       2
+#define OGMRIP_DEFAULT_VIDEO_BPP             0.25
 
 #define OGMRIP_DEFAULT_VIDEO_CAN_CROP        TRUE
 #define OGMRIP_DEFAULT_VIDEO_CAN_SCALE       TRUE
 #define OGMRIP_DEFAULT_VIDEO_MAX_WIDTH       0
 #define OGMRIP_DEFAULT_VIDEO_MAX_HEIGHT      0
-#define OGMRIP_DEFAULT_VIDEO_EXPAND          TRUE
+#define OGMRIP_DEFAULT_VIDEO_EXPAND          FALSE
 
 #define OGMRIP_DEFAULT_AUDIO_CODEC           "vorbis"
 #define OGMRIP_DEFAULT_AUDIO_QUALITY         3
@@ -157,21 +161,29 @@ G_BEGIN_DECLS
 
 #define OGMRIP_DEFAULT_SUBP_CODEC            "vobsub"
 #define OGMRIP_DEFAULT_SUBP_CHARSET          0
-#define OGMRIP_DEFAULT_SUBP_EOL              0
+#define OGMRIP_DEFAULT_SUBP_NEWLINE          0
 #define OGMRIP_DEFAULT_FORCED_SUBS           FALSE
 #define OGMRIP_DEFAULT_SPELL_CHECK           FALSE
 
 enum
 {
-  OGMRIP_ENCODING_SIZE,
-  OGMRIP_ENCODING_BITRATE,
-  OGMRIP_ENCODING_QUANTIZER
+  OGMRIP_AFTER_ENC_REMOVE,
+  OGMRIP_AFTER_ENC_KEEP,
+  OGMRIP_AFTER_ENC_UPDATE,
+  OGMRIP_AFTER_ENC_ASK
 };
 
-GType ogmrip_gconf_get_container_type   (const gchar *name);
-GType ogmrip_gconf_get_video_codec_type (const gchar *name);
-GType ogmrip_gconf_get_audio_codec_type (const gchar *name);
-GType ogmrip_gconf_get_subp_codec_type  (const gchar *name);
+void  ogmrip_gconf_init   (void);
+void  ogmrip_gconf_uninit (void);
+
+GType ogmrip_gconf_get_container_type   (const gchar *section,
+                                         const gchar *name);
+GType ogmrip_gconf_get_video_codec_type (const gchar *section,
+                                         const gchar *name);
+GType ogmrip_gconf_get_audio_codec_type (const gchar *section,
+                                         const gchar *name);
+GType ogmrip_gconf_get_subp_codec_type  (const gchar *section,
+                                         const gchar *name);
 
 G_END_DECLS
 
