@@ -605,13 +605,10 @@ packet_queue_insert(packet_queue_t *queue)
 
 typedef struct {
     unsigned int palette[16];
-    unsigned int cuspal[4];
     int delay;
-    unsigned int custom;
     unsigned int have_palette;
     unsigned int orig_frame_width, orig_frame_height;
     unsigned int origin_x, origin_y;
-    unsigned int forced_subs;
     /* index */
     packet_queue_t *spu_streams;
     unsigned int spu_streams_size;
@@ -699,23 +696,23 @@ vobsub_parse_id(vobsub_t *vob, const char *line)
     size_t idlen;
     const char *p, *q;
     p  = line;
-    while (isspace(*p))
+    while (isspace((int) *p))
 	++p;
     q = p;
-    while (isalpha(*q))
+    while (isalpha((int) *q))
 	++q;
     idlen = q - p;
     if (idlen == 0)
 	return -1;
     ++q;
-    while (isspace(*q))
+    while (isspace((int) *q))
 	++q;
     if (strncmp("index:", q, 6))
 	return -1;
     q += 6;
-    while (isspace(*q))
+    while (isspace((int) *q))
 	++q;
-    if (!isdigit(*q))
+    if (!isdigit((int) *q))
 	return -1;
     return vobsub_add_id(vob, p, idlen, atoi(q));
 }
@@ -727,10 +724,10 @@ vobsub_parse_timestamp(vobsub_t *vob, const char *line)
     const char *p;
     int h, m, s, ms;
     off_t filepos;
-    while (isspace(*line))
+    while (isspace((int) *line))
 	++line;
     p = line;
-    while (isdigit(*p))
+    while (isdigit((int) *p))
 	++p;
     if (p - line != 2)
 	return -1;
@@ -738,7 +735,7 @@ vobsub_parse_timestamp(vobsub_t *vob, const char *line)
     if (*p != ':')
 	return -1;
     line = ++p;
-    while (isdigit(*p))
+    while (isdigit((int) *p))
 	++p;
     if (p - line != 2)
 	return -1;
@@ -746,7 +743,7 @@ vobsub_parse_timestamp(vobsub_t *vob, const char *line)
     if (*p != ':')
 	return -1;
     line = ++p;
-    while (isdigit(*p))
+    while (isdigit((int) *p))
 	++p;
     if (p - line != 2)
 	return -1;
@@ -754,7 +751,7 @@ vobsub_parse_timestamp(vobsub_t *vob, const char *line)
     if (*p != ':')
 	return -1;
     line = ++p;
-    while (isdigit(*p))
+    while (isdigit((int) *p))
 	++p;
     if (p - line != 3)
 	return -1;
@@ -762,44 +759,27 @@ vobsub_parse_timestamp(vobsub_t *vob, const char *line)
     if (*p != ',')
 	return -1;
     line = p + 1;
-    while (isspace(*line))
+    while (isspace((int) *line))
 	++line;
     if (strncmp("filepos:", line, 8))
 	return -1;
     line += 8;
-    while (isspace(*line))
+    while (isspace((int) *line))
 	++line;
-    if (! isxdigit(*line))
+    if (! isxdigit((int) *line))
 	return -1;
     filepos = strtol(line, NULL, 16);
     return vobsub_add_timestamp(vob, filepos, vob->delay + ms + 1000 * (s + 60 * (m + 60 * h)));
 }
 
-static int
-vobsub_parse_size(vobsub_t *vob, const char *line)
-{
-    // size: WWWxHHH
-    char *p;
-    while (isspace(*line))
-	++line;
-    if (!isdigit(*line))
-	return -1;
-    vob->orig_frame_width = strtoul(line, &p, 10);
-    if (*p != 'x')
-	return -1;
-    ++p;
-    vob->orig_frame_height = strtoul(p, NULL, 10);
-    return 0;
-}
-
 static int
 vobsub_parse_origin(vobsub_t *vob, const char *line)
 {
     // org: X,Y
     char *p;
-    while (isspace(*line))
+    while (isspace((int) *line))
 	++line;
-    if (!isdigit(*line))
+    if (!isdigit((int) *line))
 	return -1;
     vob->origin_x = strtoul(line, &p, 10);
     if (*p != ',')
@@ -812,8 +792,8 @@ vobsub_parse_origin(vobsub_t *vob, const char *line)
 unsigned int vobsub_palette_to_yuv(unsigned int pal)
 {
     int r, g, b, y, u, v;
-    // Palette in idx file is not rgb value, it was calculated by wrong forumla.
-    // Here's reversed forumla of the one used to generate palette in idx file.
+    // Palette in idx file is not rgb value, it was calculated by wrong formula.
+    // Here's reversed formula of the one used to generate palette in idx file.
     r = pal >> 16 & 0xff;
     g = pal >> 8 & 0xff;
     b = pal & 0xff;
@@ -836,86 +816,6 @@ unsigned int vobsub_rgb_to_yuv(unsigned int rgb)
     return y << 16 | u << 8 | v;
 }
 
-static int
-vobsub_parse_palette(vobsub_t *vob, const char *line)
-{
-    // palette: XXXXXX, XXXXXX, XXXXXX, XXXXXX, XXXXXX, XXXXXX, XXXXXX, XXXXXX, XXXXXX, XXXXXX, XXXXXX, XXXXXX, XXXXXX, XXXXXX, XXXXXX, XXXXXX
-    unsigned int n;
-    n = 0;
-    while (1) {
-	const char *p;
-	int tmp;
-	while (isspace(*line))
-	    ++line;
-	p = line;
-	while (isxdigit(*p))
-	    ++p;
-	if (p - line != 6)
-	    return -1;
-	tmp = strtoul(line, NULL, 16);
-	vob->palette[n++] = vobsub_palette_to_yuv(tmp);
-	if (n == 16)
-	    break;
-	if (*p == ',')
-	    ++p;
-	line = p;
-    }
-    vob->have_palette = 1;
-    return 0;
-}
-
-static int
-vobsub_parse_custom(vobsub_t *vob, const char *line)
-{
-    //custom colors: OFF/ON(0/1)
-    if ((strncmp("ON", line + 15, 2) == 0)||strncmp("1", line + 15, 1) == 0)
-        vob->custom=1;
-    else if ((strncmp("OFF", line + 15, 3) == 0)||strncmp("0", line + 15, 1) == 0)
-        vob->custom=0;
-    else
-        return -1;
-    return 0;
-}
-
-static int
-vobsub_parse_cuspal(vobsub_t *vob, const char *line)
-{
-    //colors: XXXXXX, XXXXXX, XXXXXX, XXXXXX
-    unsigned int n, tmp;
-    n = 0;
-    line += 40;
-    while(1){
-    	const char *p;
-	while (isspace(*line))
-	    ++line;
-	p=line;
-	while (isxdigit(*p))
-	    ++p;
-	if (p - line !=6)
-	    return -1;
-	tmp = strtoul(line, NULL, 16);
-	vob->cuspal[n++] |= vobsub_rgb_to_yuv(tmp);
-	if (n==4)
-	    break;
-	if(*p == ',')
-	    ++p;
-	line = p;
-    }
-    return 0;
-}
-
-static int
-vobsub_parse_tridx(vobsub_t *vob, const char *line)
-{
-    //tridx: XXXX
-    int tridx, i;
-    tridx = strtoul((line + 26), NULL, 2);
-    for (i = 0; i < 4; ++i)
-        if ((tridx << i) & 0x08)
-            vob->cuspal[i] |= 1 << 31;
-    return tridx;
-}
-
 static int
 vobsub_parse_delay(vobsub_t *vob, const char *line)
 {
@@ -951,27 +851,8 @@ vobsub_set_lang(const char *line)
 }
 
 static int
-vobsub_parse_forced_subs(vobsub_t *vob, const char *line)
-{
-    const char *p;
-
-    p  = line;
-    while (isspace(*p))
-	++p;
-
-    if (strncasecmp("on",p,2) == 0){
-	    vob->forced_subs=~0;
-	    return 0;
-    } else if (strncasecmp("off",p,3) == 0){
-	    vob->forced_subs=0;
-	    return 0;
-    }
-	
-    return -1;
-}
-
-static int
-vobsub_parse_one_line(vobsub_t *vob, rar_stream_t *fd)
+vobsub_parse_one_line(vobsub_t *vob, rar_stream_t *fd,
+                      unsigned char **extradata, unsigned int *extradata_len)
 {
     ssize_t line_size;
     int res = -1;
@@ -979,9 +860,16 @@ vobsub_parse_one_line(vobsub_t *vob, rar_stream_t *fd)
 	char *line = NULL;
     do {
 	line_size = vobsub_getline(&line, &line_reserve, fd);
-	if (line_size < 0) {
+	if (line_size < 0 || line_size > 1000000 ||
+	    *extradata_len+line_size > 10000000) {
 	    break;
 	}
+
+	*extradata = realloc(*extradata, *extradata_len+line_size+1);
+	memcpy(*extradata+*extradata_len, line, line_size);
+	*extradata_len += line_size;
+	(*extradata)[*extradata_len] = 0;
+
 	if (*line == 0 || *line == '\r' || *line == '\n' || *line == '#')
 	    continue;
 	else if (strncmp("langidx:", line, 8) == 0)
@@ -990,19 +878,10 @@ vobsub_parse_one_line(vobsub_t *vob, rar_stream_t *fd)
 	    res = vobsub_parse_delay(vob, line);
 	else if (strncmp("id:", line, 3) == 0)
 	    res = vobsub_parse_id(vob, line + 3);
-	else if (strncmp("palette:", line, 8) == 0)
-	    res = vobsub_parse_palette(vob, line + 8);
-	else if (strncmp("size:", line, 5) == 0)
-	    res = vobsub_parse_size(vob, line + 5);
 	else if (strncmp("org:", line, 4) == 0)
 	    res = vobsub_parse_origin(vob, line + 4);
 	else if (strncmp("timestamp:", line, 10) == 0)
 	    res = vobsub_parse_timestamp(vob, line + 10);
-	else if (strncmp("custom colors:", line, 14) == 0)
-	    //custom colors: ON/OFF, tridx: XXXX, colors: XXXXXX, XXXXXX, XXXXXX,XXXXXX
-	    res = vobsub_parse_cuspal(vob, line) + vobsub_parse_tridx(vob, line) + vobsub_parse_custom(vob, line);
-	else if (strncmp("forced subs:", line, 12) == 0)
-		res = vobsub_parse_forced_subs(vob, line + 12);
 	else {
 	    mp_msg(MSGT_VOBSUB,MSGL_V, "vobsub: ignoring %s", line);
 	    continue;
@@ -1089,6 +968,8 @@ vobsub_parse_ifo(void* this, const char *const name, unsigned int *palette, unsi
 void *
 vobsub_open(const char *const name,const char *const ifo,const int force,void** spu)
 {
+    unsigned char *extradata = NULL;
+    unsigned int extradata_len = 0;
     vobsub_t *vob = calloc(1, sizeof(vobsub_t));
     if(spu)
       *spu = NULL;
@@ -1120,15 +1001,14 @@ vobsub_open(const char *const name,const char *const ifo,const int force,void**
 		  return NULL;
 		}
 	    } else {
-		while (vobsub_parse_one_line(vob, fd) >= 0)
+	        while (vobsub_parse_one_line(vob, fd, &extradata, &extradata_len) >= 0)
 		    /* NOOP */ ;
 		rar_close(fd);
 	    }
-	    /* if no palette in .idx then use custom colors */
-	    if ((vob->custom == 0)&&(vob->have_palette!=1))
-		vob->custom = 1;
-	    if (spu && vob->orig_frame_width && vob->orig_frame_height)
-	      *spu = spudec_new_scaled_vobsub(vob->palette, vob->cuspal, vob->custom, vob->orig_frame_width, vob->orig_frame_height);
+	    if (spu)
+	      *spu = spudec_new_scaled(vob->palette, vob->orig_frame_width, vob->orig_frame_height, extradata, extradata_len);
+	    if (extradata)
+	      free(extradata);
 
 	    /* read the indexed mpeg_stream */
 	    strcpy(buf, name);
@@ -1261,15 +1141,6 @@ int vobsub_get_index_by_id(void *vobhandle, int id)
     return j;
 }
 
-unsigned int 
-vobsub_get_forced_subs_flag(void const * const vobhandle)
-{
-  if (vobhandle)
-    return ((vobsub_t*) vobhandle)->forced_subs;
-  else
-    return 0;
-}
-
 int
 vobsub_set_from_lang(void *vobhandle, unsigned char * lang)
 {
